"""
Test cases for the L{xmantissa.webadmin} module.
"""


from twisted.trial.unittest import TestCase


from nevow.athena import LivePage
from nevow.context import WovenContext
from nevow.testutil import FakeRequest
from nevow.loaders import stan
from nevow.tags import html, head, body, directive
from nevow.inevow import IRequest

from axiom.store import Store
from axiom.userbase import LoginSystem, LoginMethod
from axiom.dependency import installOn

from axiom.plugins.mantissacmd import Mantissa

from xmantissa.webadmin import (
    LocalUserBrowser, LocalUserBrowserFragment,
    UserInteractionFragment, EndowFragment, DepriveFragment,
    SuspendFragment, UnsuspendFragment)

from xmantissa.product import Product


class UserInteractionFragmentTestCase(TestCase):
    def setUp(self):
        """
        Create a site store and a user store with a L{LocalUserBrowser}
        installed on it.
        """
        self.siteStore = Store()
        self.loginSystem = LoginSystem(store=self.siteStore)
        installOn(self.loginSystem, self.siteStore)

        self.userStore = Store()
        self.userStore.parent = self.siteStore
        self.browser = LocalUserBrowser(store=self.userStore)


    def test_createUser(self):
        """
        Test that L{webadmin.UserInteractionFragment.createUser} method
        actually creates a user.
        """
        userInteractionFragment = UserInteractionFragment(self.browser)
        userInteractionFragment.createUser(
            u'testuser', u'localhost', u'password')

        account = self.loginSystem.accountByAddress(u'testuser', u'localhost')
        self.assertEquals(account.password, u'password')


    def test_rendering(self):
        """
        Test that L{webadmin.UserInteractionFragment} renders without raising
        any exceptions.
        """
        f = UserInteractionFragment(self.browser)

        p = LivePage(
            docFactory=stan(
                html[
                    head(render=directive('liveglue')),
                    body(render=lambda ctx, data: f)]))
        f.setFragmentParent(p)

        ctx = WovenContext()
        req = FakeRequest()
        ctx.remember(req, IRequest)

        d = p.renderHTTP(ctx)
        def rendered(ign):
            p.action_close(None)
        d.addCallback(rendered)
        return d



class ActionsTestCase(TestCase):
    """
    Tests to verify that actions behave as expected.

    @ivar siteStore: A site store containing an administrative user's account.

    @ivar siteAccount: The L{axiom.userbase.LoginAccount} for the
    administrator, in the site store.

    @ivar siteMethod: The single L{axiom.userbase.LoginMethod} for the
    administrator, in the site store.

    @ivar localUserBrowserFragment: A L{LocalUserBrowserFragment} examining the
    administrator's L{LocalUserBrowser} powerup.
    """

    def setUp(self):
        """
        Construct a site and user store with an administrator that can invoke the
        web administrative tools, setting the instance variables described in
        this class's docstring.
        """
        self.siteStore = Store(filesdir=self.mktemp())
        Mantissa().installSite(self.siteStore, u"localhost", u"", False)
        Mantissa().installAdmin(self.siteStore, u'admin', u'localhost', u'asdf')

        self.siteMethod = self.siteStore.findUnique(
            LoginMethod, LoginMethod.localpart == u'admin')
        self.siteAccount = self.siteMethod.account
        userStore = self.siteAccount.avatars.open()
        lub = userStore.findUnique(LocalUserBrowser)
        self.localUserBrowserFragment = LocalUserBrowserFragment(lub)


    def test_actionTypes(self):
        """
        Verify that all the action methods expose the appropriate fragment
        objects, with their attributes set to indicate the correct objects to
        manipulate.
        """
        myRowID = self.localUserBrowserFragment.linkToItem(self.siteMethod)
        actionMap = [('installOn', EndowFragment),
                     ('uninstallFrom', DepriveFragment),
                     ('suspend', SuspendFragment),
                     ('unsuspend', UnsuspendFragment)]
        for action, fragmentType in actionMap:
            resultFragment = self.localUserBrowserFragment.performAction(
                action, myRowID)
            self.failUnless(isinstance(resultFragment, fragmentType),
                            "%s does not return a %s" %
                            (action, fragmentType))
            self.assertEquals(resultFragment.fragmentParent,
                              self.localUserBrowserFragment)
            self.assertEquals(resultFragment.account, self.siteAccount)



class RenderingTestCase(TestCase):
    """
    Test cases for HTML rendering of various fragments.
    """

    def doRendering(self, fragmentClass):
        """
        Verify that the given fragment class will render without raising an
        exception.
        """
        siteStore = Store()

        loginSystem = LoginSystem(store=siteStore)
        installOn(loginSystem, siteStore)
        p = Product(store=siteStore, types=["xmantissa.webadmin.LocalUserBrowser",
                                            "xmantissa.signup.SignupConfiguration"])
        account = loginSystem.addAccount(u'testuser', u'localhost', None)
        p.installProductOn(account.avatars.open())
        f = fragmentClass(None, u'testuser', account)

        p = LivePage(
            docFactory=stan(
                html[
                    head(render=directive('liveglue')),
                    body(render=lambda ctx, data: f)]))
        f.setFragmentParent(p)

        ctx = WovenContext()
        req = FakeRequest()
        ctx.remember(req, IRequest)

        d = p.renderHTTP(ctx)
        def rendered(ign):
            p.action_close(None)
        d.addCallback(rendered)
        return d


    def test_endowRendering(self):
        """
        Verify that L{EndowFragment} can render without raising an exception.
        """
        return self.doRendering(EndowFragment)


    def test_depriveRendering(self):
        """
        Verify that L{DepriveFragment} can render without raising an exception.
        """
        return self.doRendering(DepriveFragment)


    def test_suspendRendering(self):
        """
        Verify that L{SuspendFragment} can render without raising an exception.
        """
        return self.doRendering(SuspendFragment)


    def test_unsuspendRendering(self):
        """
        Verify that L{UnsuspendFragment} can render without raising an
        exception.
        """
        return self.doRendering(UnsuspendFragment)





# -*- test-case-name: xmantissa.test.historic.test_searchresult1to2 -*-

from axiom.test.historic.stubloader import saveStub

from xmantissa.search import SearchResult

def createDatabase(s):
    SearchResult(store=s, indexedItem=s, identifier=0)

if __name__ == '__main__':
    saveStub(createDatabase, 7976)




from axiom.test.historic import stubloader

from xmantissa.ixmantissa import IWebViewer
from xmantissa.publicweb import CustomizedPublicPage
from xmantissa.webapp import PrivateApplication
from xmantissa.website import WebSite
from xmantissa.webgestalt import AuthenticationApplication
from xmantissa.prefs import PreferenceAggregator, DefaultPreferenceCollection
from xmantissa.search import SearchAggregator


class PATestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        """
        Ensure upgraded fields refer to correct items.
        """

        pa = self.store.findUnique(PrivateApplication)

        self.assertEqual(pa.customizedPublicPage,
                         pa.store.findUnique(CustomizedPublicPage))
        self.assertEqual(pa.authenticationApplication,
                         pa.store.findUnique(AuthenticationApplication))
        self.assertEqual(pa.preferenceAggregator,
                         pa.store.findUnique(PreferenceAggregator))
        self.assertEqual(pa.defaultPreferenceCollection,
                         pa.store.findUnique(DefaultPreferenceCollection))
        self.assertEqual(pa.searchAggregator,
                         pa.store.findUnique(SearchAggregator))
        self.assertEqual(pa.website,
                         pa.store.findUnique(WebSite))

    def test_privateKeyDoesntChange(self):
        """
        Verify that upgrading the PrivateApplication does not change its privateKey
        attribute.
        """
        pa = self.store.findUnique(PrivateApplication)
        oldStore = self.openLegacyStore()
        oldPA = oldStore.getItemByID(pa.storeID, autoUpgrade=False)

        self.assertEqual(oldPA.privateKey, pa.privateKey)


    def test_webViewer(self):
        """
        At version 5, L{PrivateApplication} should be an
        L{IWebViewer} powerup on its store.
        """
        application = self.store.findUnique(PrivateApplication)
        interfaces = list(application.store.interfacesFor(application))
        self.assertIn(IWebViewer, interfaces)




"""
Tests for the upgrade of L{MessageQueue} from version 1 to 2, in which its
C{scheduler} attribute was removed.
"""

from axiom.test.historic.stubloader import StubbedTest

from xmantissa.ixmantissa import IMessageRouter
from xmantissa.interstore import MessageQueue

from xmantissa.test.historic.stub_messagequeue1to2 import MESSAGE_COUNT


class MessageQueueUpgradeTests(StubbedTest):
    def test_attributes(self):
        """
        The value of the C{messageCounter} attribute is preserved by the
        upgrade.
        """
        self.assertEquals(
            self.store.findUnique(MessageQueue).messageCounter,
            MESSAGE_COUNT)

    def test_powerup(self):
        """
        The L{MessageQueue} is still a L{IMessageRouter} powerup on its store
        after the upgrade.
        """
        self.assertEquals(
            [self.store.findUnique(MessageQueue)],
            list(self.store.powerupsFor(IMessageRouter)))





from axiom.test.historic import stubloader
from xmantissa.fulltext import PyLuceneIndexer

class PyLuceneIndexerTestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        index = self.store.findUnique(PyLuceneIndexer)
        self.assertEqual(index.indexDirectory, 'foo.index')
        # we called reset(), and there are no indexed items
        self.assertEqual(index.indexCount, 0)
        self.assertEqual(index.installedOn, self.store)




"""
An interactive demonstration of L{xmantissa.liveform.LiveForm} and
L{xmantissa.liveform.ListChangeParameter}.

Run this test like this::
    $ twistd -n athena-widget --element=xmantissa.test.acceptance.liveform.testname
    $ firefox http://localhost:8080/
    (where testname is one of "coerce", "inputerrors",
     "listChangeParameter", "listChangeParameterCompact",
     "listChangeParameterNoDefaults", "choiceParameter",
     "choiceParameterCompact")

This will display a form which rejects most inputs.
"""

from xmantissa.liveform import (TEXT_INPUT, InputError, Parameter, LiveForm,
    ListChangeParameter, ChoiceParameter, Option)


def coerce(theText):
    """
    Reject all values of C{theText} except C{'hello, world'}.
    """
    if theText != u'hello, world':
        raise InputError(u"Try entering 'hello, world'")


def inputerrors():
    """
    Create a L{LiveForm} which rejects most inputs in order to demonstrate how
    L{InputError} is handled in the browser.
    """
    form = LiveForm(
        lambda theText: None,
        [Parameter(u'theText', TEXT_INPUT, coerce, 'Some Text')],
        u'LiveForm input errors acceptance test',
        )
    return form


_parameterDefaults = [{u'foo': 1,  u'bar': 2,  u'baz': ['1']},
                      {u'foo': 10, u'bar': 20, u'baz': ['2']}]


def _listChangeParameter(**parameterKwargs):
    counter = [0]
    def theCallable(repeatableFoo):
        for create in repeatableFoo.create:
            create.setter(u'other thing %d' % (counter[0],))
            counter[0] += 1
        return u'Created %s, edited %s, deleted %s' % (repeatableFoo.create,
                                                       repeatableFoo.edit,
                                                       repeatableFoo.delete)
    form = LiveForm(
        theCallable,
        [ListChangeParameter(
            u'repeatableFoo',
            [Parameter('foo', TEXT_INPUT, int, 'Enter a number'),
             Parameter('bar', TEXT_INPUT, int, 'And another'),
             ChoiceParameter(
                 'baz',
                 [Option('Value 1', '1', True),
                  Option('Value 2', '2', False)],
                 'Pick something')],
            modelObjectDescription=u'Repeatable Foo',
            **parameterKwargs)])
    form.jsClass = u'Mantissa.Test.EchoingFormWidget'
    return form



def listChangeParameter():
    """
    Create a L{LiveForm} with a L{ListChangeParameter}.
    """
    return _listChangeParameter(
        defaults=_parameterDefaults,
        modelObjects=(u'the first thing', u'the second thing'))



def listChangeParameterCompact():
    """
    Create a compact L{LiveForm} with a L{ListChangeParameter}.
    """
    liveForm = listChangeParameter()
    liveForm.compact()
    return liveForm



def listChangeParameterNoDefaults():
    """
    Create a L{LiveForm} with a L{ListChangeParameter} and no defaults.
    """
    return _listChangeParameter(defaults=[], modelObjects=[])



def choiceParameter():
    """
    Create a L{LiveForm} with a L{ChoiceParameter}.
    """
    return LiveForm(
        lambda **k: unicode(k),
        [ChoiceParameter(
            'choice',
            [Option('Thing 1', 'thing-one', False),
             Option('Thing 2', 'thing-two', True),
             Option('Thing 3', 'thing-three', False)],
            'This is a choice between things')])



def choiceParameterCompact():
    """
    Compact version of the form returned by L{choiceParameter}.
    """
    liveForm = choiceParameter()
    liveForm.compact()
    return liveForm



# -*- test-case-name: xmantissa.test.test_smtp -*-

"""
An RFC 2821 address parser.
"""

import inspect, re

from twisted.internet import address

from xmantissa.error import (
    AddressTooLong, InvalidAddress, InvalidTrailingBytes)


class Address(object):
    """
    An RFC 2821 path.

    @ivar route: C{None} or a C{list} of C{str} giving the source-specified
    route of this message.  This is obsolete and should not be respected,
    but is made available for completeness.

    @ivar localpart: C{None} or a C{str} giving the local part of this address.

    @ivar domain: C{None} or a C{str}, L{IPv4Address} or L{IPv6Address}
    giving the domain part of this address.
    """
    route = localpart = domain = None

    def __init__(self, route, localpart, domain):
        assert (localpart is None) == (domain is None)

        self.route = route
        self.localpart = localpart
        self.domain = domain


    def __eq__(self, other):
        """
        Compare this address to another for equality.

        Two addresses are equal if their route, localpart, and domain
        attributes are equal.  Comparison against non-Address objects is
        unimplemented.
        """
        if isinstance(other, Address):
            a = (self.route, self.localpart, self.domain)
            b = (other.route, other.localpart, other.domain)
            return a == b
        return NotImplemented


    def __ne__(self, other):
        """
        Compare this address to another for inequality.

        See L{__eq__}.
        """
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result


    def __str__(self):
        """
        Format this address as an RFC 2821 path string.
        """
        mailbox = ''
        if self.localpart is not None:
            mailbox = '%s@%s' % (self.localpart, self.domain)
        route = ''
        if self.route is not None:
            route = '@' + ',@'.join(self.route) + ':'
        return '<%s%s>' % (route, mailbox)


    def __repr__(self):
        return 'Address(%r, %r, %r)' % (self.route, self.localpart, self.domain)



class _AddressParser(object):
    def f(s):
        d = inspect.currentframe().f_back.f_locals
        return "(?:" + (s % d) + ")"

    alpha = 'a-zA-Z'
    digit = '0-9'
    hexdigit = 'a-fA-F0-9'

    letDig = f(r"[%(alpha)s%(digit)s]")
    ldhStr = f(r"[%(alpha)s%(digit)s-]*%(letDig)s")

    sNum = f(r"[%(digit)s]{1,3}")
    ipv4AddressLiteral = f(r"%(sNum)s(?:\.%(sNum)s){3}")

    ipv6Hex = f(r"[%(hexdigit)s]")
    ipv6Full = f(r"%(ipv6Hex)s(?::%(ipv6Hex)s){7}")
    ipv6Comp = f(r"(?:%(ipv6Hex)s(?::%(ipv6Hex)s){0,5})?::(?:%(ipv6Hex)s(?::%(ipv6Hex)s){0,5})?")

    ipv6v4Full = f(r"%(ipv6Hex)s(?::%(ipv6Hex)s){5}:%(ipv4AddressLiteral)s")
    ipv6v4Comp = f(r"(?:%(ipv6Hex)s(?::%(ipv6Hex)s){0,3})?::(?:%(ipv6Hex)s(?::%(ipv6Hex)s){0,3})?%(ipv4AddressLiteral)s")

    ipv6Address = f(r"%(ipv6Full)s|%(ipv6Comp)s|%(ipv6v4Full)s|%(ipv6v4Comp)s")
    ipv6AddressLiteral = f(r"IPv6:%(ipv6Address)s")

    standardizedTag = ldhStr

    noWsCtl = '\x01-\x08\x0B\x0C\x0E-\x7F'

    somePrintableUSAscii = '\x21-\x5A\x5E-\x7E'

    text = '\x01-\x09\x0B\x0C\x0E-\x7F'
    quotedPair = f(r"\\[%(text)s]") # XXX obs-qp

    dtext = f(r"[%(noWsCtl)s\x21-\x5A\x5E-\x7E]")
    dcontent = f(r"%(dtext)s|%(quotedPair)s")

    generalAddressLiteral = f(r"%(standardizedTag)s:%(dcontent)s+")

    qtext = f(r"%(noWsCtl)s|[\x21\x23-\x5B\x5D-\x7F]")
    qcontent = f(r"%(qtext)s|%(quotedPair)s")
    quotedString = f(r"\"(?:%(qcontent)s)*\"")

    addressLiteral = f(r"%(ipv4AddressLiteral)s|%(ipv6AddressLiteral)s|%(generalAddressLiteral)s")
    addressLiteralNamed = f(
        r"(?P<ipv4Literal>%(ipv4AddressLiteral)s)|"
        r"(?P<ipv6Literal>%(ipv6AddressLiteral)s)|"
        r"(?P<generalLiteral>%(generalAddressLiteral)s)")

    subdomain = f(r"%(letDig)s(?:%(ldhStr)s)?")
    domain = f(r"(?:%(subdomain)s(?:\.%(subdomain)s)+)|%(addressLiteral)s")
    domainNamed = f(r"(?P<domain>(?:%(subdomain)s(?:\.%(subdomain)s)+)|%(addressLiteralNamed)s)")
    atDomain = f(r"@%(domain)s")

    atext = f(r"%(letDig)s|[!#$%%&'*+-/=?^_`{|}-]")
    atom = f(r"%(atext)s+")
    dotString = f(r"%(atom)s(\.%(atom)s)*")

    localPart = f(r"%(dotString)s|%(quotedString)s")

    adl = f(r"(?P<adl>%(atDomain)s(?:,%(atDomain)s)*)")
    mailbox = f(r"(?P<localPart>%(localPart)s)?(?P<at>@)%(domainNamed)s?")

    path = f(r"<(?:%(adl)s:)?(?:%(mailbox)s)?>")

    address = re.compile("^" + path + "$")

    def __call__(self, arglist, line):
        # RFC 2821 4.5.3.1
        if len(line) > 256:
            raise AddressTooLong()

        match = self.address.match(line)
        if match is None:
            raise InvalidAddress()

        d = match.groupdict()

        if d['adl']:
            route = d['adl'][1:].split(',@')
        else:
            route = None

        localpart = d['localPart']

        if d['domain']:
            domain = d['domain']
        elif d['ipv4Literal']:
            domain = address.IPv4Address(d['ipv4Literal'])
        elif d['ipv6Literal']:
            # Chop off the leading 'IPv6:'
            domain = address.IPv6Address(d['ipv6Literal'][5:])
        else:
            domain = d['generalLiteral']

        at = d['at']

        if at:
            localpart = localpart or ''
            domain = domain or ''

        arglist.append(Address(route, localpart, domain))

        return match.end()



def parseAddress(address):
    """
    Parse the given RFC 2821 email address into a structured object.

    @type address: C{str}
    @param address: The address to parse.

    @rtype: L{Address}

    @raise xmantissa.error.ArgumentError: The given string was not a valid RFC
    2821 address.
    """
    parts = []
    parser = _AddressParser()
    end = parser(parts, address)
    if end != len(address):
        raise InvalidTrailingBytes()
    return parts[0]





from zope.interface import implements

from twisted.trial import unittest
from twisted.application.service import IService
from twisted.internet.defer import gatherResults

from axiom import iaxiom, store, batch, item, attributes
from axiom.userbase import LoginSystem
from axiom.dependency import installOn
from axiom.errors import SQLError

from xmantissa import ixmantissa, fulltext


def identifiersFrom(hits):
    """
    Convert iterable of hits into list of integer unique identifiers.
    """
    return [int(h.uniqueIdentifier) for h in hits]


class IndexableThing(item.Item):
    implements(ixmantissa.IFulltextIndexable)

    _uniqueIdentifier = attributes.bytes()

    _textParts = attributes.inmemory()
    _keywordParts = attributes.inmemory()
    _documentType = attributes.inmemory()

    def uniqueIdentifier(self):
        return self._uniqueIdentifier


    def textParts(self):
        return self._textParts


    def keywordParts(self):
        return self._keywordParts


    def documentType(self):
        return self._documentType


    def sortKey(self):
        return self.uniqueIdentifier()



class FakeMessageSource(item.Item):
    """
    Stand-in for an item type returned from L{axiom.batch.processor}.  Doesn't
    actually act as a source of anything, just used to test that items are kept
    track of properly.
    """
    anAttribute = attributes.text(doc="""
    Nothing.  Axiom requires at least one attribute per item-type.
    """)

    added = attributes.inmemory()
    removed = attributes.inmemory()

    def activate(self):
        self.added = []
        self.removed = []


    def addReliableListener(self, what, style):
        self.added.append((what, style))


    def removeReliableListener(self, what):
        self.removed.append(what)



class IndexerTestsMixin:
    def createIndexer(self):
        raise NotImplementedError()


    def openWriteIndex(self):
        try:
            return self.indexer.openWriteIndex()
        except NotImplementedError, e:
            raise unittest.SkipTest(str(e))


    def openReadIndex(self):
        try:
            return self.indexer.openReadIndex()
        except NotImplementedError, e:
            raise unittest.SkipTest(str(e))


    def setUp(self):
        self.path = u'index'
        self.store = store.Store(filesdir=self.mktemp())
        self.indexer = self.createIndexer()



class FulltextTestsMixin(IndexerTestsMixin):
    """
    Tests for any IFulltextIndexer provider.
    """

    def testSources(self):
        """
        Test that multiple IBatchProcessors can be added to a RemoteIndexer and
        that an indexer can be reset, with respect to input from its sources.
        """
        firstSource = FakeMessageSource(store=self.store)
        secondSource = FakeMessageSource(store=self.store)
        self.indexer.addSource(firstSource)
        self.indexer.addSource(secondSource)

        self.assertEquals(firstSource.added, [(self.indexer, iaxiom.REMOTE)])
        self.assertEquals(secondSource.added, [(self.indexer, iaxiom.REMOTE)])

        self.assertEquals(
            list(self.indexer.getSources()),
            [firstSource, secondSource])

        firstSource.added = []
        secondSource.added = []

        self.indexer.reset()
        self.assertEquals(firstSource.removed, [self.indexer])
        self.assertEquals(secondSource.removed, [self.indexer])
        self.assertEquals(firstSource.added, [(self.indexer, iaxiom.REMOTE)])
        self.assertEquals(secondSource.added, [(self.indexer, iaxiom.REMOTE)])


    def test_emptySearch(self):
        """
        Test that a search with no term and no keywords returns an empty result
        set.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='7',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='21',
                _textParts=[u'cherry', u'drosophila melanogaster'],
                _keywordParts={}))
        writer.close()

        reader  = self.openReadIndex()
        results = list(reader.search(u'', {}))
        self.assertEquals(results, [])


    def testSimpleSerializedUsage(self):
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='7',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='21',
                _textParts=[u'cherry', u'drosophila melanogaster'],
                _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()

        results = identifiersFrom(reader.search(u'apple'))
        self.assertEquals(results, [7])

        results = identifiersFrom(reader.search(u'banana'))
        self.assertEquals(results, [7])

        results = identifiersFrom(reader.search(u'cherry'))
        self.assertEquals(results, [21])

        results = identifiersFrom(reader.search(u'drosophila'))
        self.assertEquals(results, [21])

        results = identifiersFrom(reader.search(u'melanogaster'))
        self.assertEquals(results, [21])

        reader.close()


    def testWriteReadWriteRead(self):
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='1',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()
        results = identifiersFrom(reader.search(u'apple'))
        self.assertEquals(results, [1])
        results = identifiersFrom(reader.search(u'banana'))
        self.assertEquals(results, [1])
        reader.close()

        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='2',
                _textParts=[u'cherry', u'drosophila melanogaster'],
                _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()
        results = identifiersFrom(reader.search(u'apple'))
        self.assertEquals(results, [1])
        results = identifiersFrom(reader.search(u'banana'))
        self.assertEquals(results, [1])

        results = identifiersFrom(reader.search(u'cherry'))
        self.assertEquals(results, [2])
        results = identifiersFrom(reader.search(u'drosophila'))
        self.assertEquals(results, [2])
        results = identifiersFrom(reader.search(u'melanogaster'))
        self.assertEquals(results, [2])
        reader.close()


    def testReadBeforeWrite(self):
        reader = self.openReadIndex()
        results = identifiersFrom(reader.search(u'apple'))
        self.assertEquals(results, [])


    def test_remove(self):
        """
        Test that the L{remove} method of an indexer successfully removes
        the item it is given from its index.
        """
        item = IndexableThing(
            _documentType=u'thing',
            _uniqueIdentifier='50',
            _textParts=[u'apple', u'banana'],
            _keywordParts={})
        writer = self.openWriteIndex()
        writer.add(item)
        writer.close()

        self.indexer.remove(item)
        self.indexer._flush()

        reader = self.openReadIndex()
        self.assertEqual(
            identifiersFrom(reader.search(u'apple')), [])


    def test_removalFromReadIndex(self):
        """
        Add a document to an index and then remove it, asserting that it no
        longer appears once it has been removed.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='50',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()
        reader.remove(u'50')
        reader.close()

        reader = self.openReadIndex()
        self.assertEqual(
            identifiersFrom(reader.search(u'apple')), [])


    def testKeywordIndexing(self):
        """
        Test that an L{IFulltextIndexable}'s keyword parts can be searched for.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='50',
                _textParts=[u'apple', u'banana'],
                _keywordParts={u'subject': u'fruit'}))
        writer.close()

        reader = self.openReadIndex()
        self.assertEquals(
            identifiersFrom(reader.search(u'airplane')), [])
        self.assertEquals(
            identifiersFrom(reader.search(u'fruit')), [])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple')), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple', {u'subject': u'fruit'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'subject': u'fruit'})), [50])


    def test_typeRestriction(self):
        """
        Test that the type of an IFulltextIndexable is automatically found when
        indexing and searching for items of a particular type limits the
        results appropriately.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'first',
                _uniqueIdentifier='1',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.add(IndexableThing(
                _documentType=u'second',
                _uniqueIdentifier='2',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()
        self.assertEquals(
            identifiersFrom(reader.search(u'apple', {'documentType': u'first'})),
            [1])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple', {'documentType': u'second'})),
            [2])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple', {'documentType': u'three'})),
            [])


    def testKeywordTokenization(self):
        """
        Keyword values should be tokenized just like text parts.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='50',
                _textParts=[u'apple', u'banana'],
                _keywordParts={u'subject': u'list of fruit things'}))
        writer.close()

        reader = self.openReadIndex()
        self.assertEquals(
            identifiersFrom(reader.search(u'pear')), [])
        self.assertEquals(
            identifiersFrom(reader.search(u'fruit')), [])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple')), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'apple', {u'subject': u'fruit'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'subject': u'fruit'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'subject': u'list'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'subject': u'things'})), [50])


    def testKeywordCombination(self):
        """
        Multiple keyword searches should be AND'ed
        """
        writer = self.openWriteIndex()
        def makeIndexable(uniqueIdentifier, **k):
            writer.add(IndexableThing(
                         _documentType=u'thing',
                         _uniqueIdentifier=str(uniqueIdentifier),
                         _textParts=[],
                         _keywordParts=dict((unicode(k), unicode(v))
                                                for (k, v) in k.iteritems())))

        makeIndexable(50, name='john', car='honda')
        makeIndexable(51, name='john', car='mercedes')
        writer.close()

        reader = self.openReadIndex()

        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'car': u'honda'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'car': u'mercedes'})), [51])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'name': u'john'})), [50, 51])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'name': u'john', u'car': u'honda'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'name': u'john', u'car': u'mercedes'})), [51])


    def testKeywordValuesInPhrase(self):
        """
        Keyword values should return results when included in the main phrase
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                    _documentType=u'thing',
                    _uniqueIdentifier='50',
                    _textParts=[u'my name is jack'],
                    _keywordParts={u'car': u'honda'}))
        writer.close()

        reader = self.openReadIndex()

        self.assertEquals(
            identifiersFrom(reader.search(u'honda', {})), [])
        self.assertEquals(
            identifiersFrom(reader.search(u'jack', {})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'car': u'honda'})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'', {u'car': u'jack'})), [])

    def testDigitSearch(self):
        """
        Should get results if we search for digits that appear in indexed
        documents
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                    _documentType=u'thing',
                    _uniqueIdentifier='50',
                    _textParts=[u'123 456'],
                    _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()

        self.assertEquals(
            identifiersFrom(reader.search(u'123', {})), [50])
        self.assertEquals(
            identifiersFrom(reader.search(u'456', {})), [50])

    def testSorting(self):
        """
        Index some stuff with out of order sort keys and ensure that
        they come back ordered by sort key.
        """

        writer = self.openWriteIndex()
        keys = (5, 20, 6, 127, 2)

        for k in keys:
            writer.add(IndexableThing(
                    _documentType=u'thing',
                    _uniqueIdentifier=str(k),
                    _textParts=[u'ok'],
                    _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()

        self.assertEquals(identifiersFrom(reader.search(u'ok')),
                          list(sorted(keys)))

    def testSortAscending(self):
        """
        Test that the C{sortAscending} parameter to C{search} is observed
        """
        writer = self.openWriteIndex()

        for i in xrange(5):
            writer.add(IndexableThing(
                        _documentType=u'thing',
                        _uniqueIdentifier=str(i),
                        _textParts=[u'ok'],
                        _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()

        self.assertEquals(identifiersFrom(reader.search(u'ok')), range(5))
        self.assertEquals(identifiersFrom(reader.search(u'ok', sortAscending=True)), range(5))
        self.assertEquals(identifiersFrom(reader.search(u'ok', sortAscending=False)), range(4, -1, -1))


class CorruptionRecoveryMixin(IndexerTestsMixin):
    def corruptIndex(self):
        raise NotImplementedError()


    def testRecoveryAfterFailure(self):
        """
        Create an indexer, attach some sources to it, let it process some
        messages, corrupt the database, let it try to clean things up, then
        make sure the index is in a reasonable state.
        """
        # Try to access the indexer directly first so that if it is
        # unavailable, the test will be skipped.
        self.openReadIndex().close()

        service = batch.BatchProcessingService(self.store, iaxiom.REMOTE)
        task = service.step()

        source = batch.processor(IndexableThing)(store=self.store)
        self.indexer.addSource(source)

        things = [
            IndexableThing(store=self.store,
                           _documentType=u'thing',
                           _uniqueIdentifier='100',
                           _textParts=[u'apple', u'banana'],
                           _keywordParts={}),
            IndexableThing(store=self.store,
                           _documentType=u'thing',
                           _uniqueIdentifier='200',
                           _textParts=[u'cherry'],
                           _keywordParts={})]

        for i in xrange(len(things)):
            task.next()

        self.indexer.suspend()

        # Sanity check - make sure both items come back from a search before
        # going on with the real core of the test.
        reader = self.openReadIndex()
        self.assertEquals(identifiersFrom(reader.search(u'apple')), [100])
        self.assertEquals(identifiersFrom(reader.search(u'cherry')), [200])
        self.assertEquals(identifiersFrom(reader.search(u'drosophila')), [])
        reader.close()

        self.corruptIndex()
        self.indexer.resume()

        things.append(
            IndexableThing(store=self.store,
                           _documentType=u'thing',
                           _uniqueIdentifier='300',
                           _textParts=[u'drosophila', u'melanogaster'],
                           _keywordParts={}))

        # Step it once so that it notices the index has been corrupted.
        task.next()
        self.indexer.suspend()

        # At this point, the index should have been deleted, so any search
        # should turn up no results.
        reader = self.openReadIndex()
        self.assertEquals(identifiersFrom(reader.search(u'apple')), [])
        self.assertEquals(identifiersFrom(reader.search(u'cherry')), [])
        self.assertEquals(identifiersFrom(reader.search(u'drosophila')), [])
        reader.close()

        self.indexer.resume()

        # Step it another N so that each thing gets re-indexed.
        for i in xrange(len(things)):
            task.next()

        self.indexer.suspend()

        reader = self.openReadIndex()
        self.assertEquals(identifiersFrom(reader.search(u'apple')), [100])
        self.assertEquals(identifiersFrom(reader.search(u'cherry')), [200])
        self.assertEquals(identifiersFrom(reader.search(u'drosophila')), [300])
        reader.close()



class HypeTestsMixin:
    def createIndexer(self):
        return fulltext.HypeIndexer(store=self.store, indexDirectory=self.path)



class HypeFulltextTestCase(HypeTestsMixin, FulltextTestsMixin, unittest.TestCase):
    skip = "These tests don't actually pass - and I don't even care."



class PyLuceneTestsMixin:
    def createIndexer(self):
        return fulltext.PyLuceneIndexer(store=self.store, indexDirectory=self.path)


class PyLuceneFulltextTestCase(PyLuceneTestsMixin, FulltextTestsMixin, unittest.TestCase):
    def testAutomaticClosing(self):
        """
        Test that if we create a writer and call the close-helper function,
        the writer gets closed.
        """
        writer = self.openWriteIndex()
        fulltext._closeIndexes()
        self.failUnless(writer.closed, "Writer should have been closed.")


    def testRepeatedClosing(self):
        """
        Test that if for some reason a writer is explicitly closed after the
        close-helper has run, nothing untoward occurs.
        """
        writer = self.openWriteIndex()
        fulltext._closeIndexes()
        writer.close()
        self.failUnless(writer.closed, "Writer should have stayed closed.")

    def test_resultSlicing(self):
        """
        Test that the wrapper object return by the pylucene index correctly
        handles slices
        """

        writer = self.openWriteIndex()
        identifiers = range(20)
        for i in identifiers:
            writer.add(IndexableThing(
                        _documentType=u'thing',
                        _uniqueIdentifier=str(i),
                        _textParts=[u'e'],
                        _keywordParts={}))
        writer.close()

        reader = self.openReadIndex()

        results = reader.search(u'e')

        self.assertEquals(identifiersFrom(results), identifiers)
        self.assertEquals(identifiersFrom(results[0:None:2]), identifiers[0:None:2])
        self.assertEquals(identifiersFrom(results[0:5:1]), identifiers[0:5:1])
        self.assertEquals(identifiersFrom(results[15:0:-1]), identifiers[15:0:-1])
        self.assertEquals(identifiersFrom(results[15:None:-1]), identifiers[15:None:-1])
        self.assertEquals(identifiersFrom(results[0:24:2]), identifiers[0:24:2])
        self.assertEquals(identifiersFrom(results[24:None:-1]), identifiers[24:None:-1])


    def test_hitWrapperAttributes(self):
        """
        Test that L{xmantissa.fulltext._PyLuceneHitWrapper}'s attributes are
        set correctly
        """
        class Indexable:
            implements(ixmantissa.IFulltextIndexable)

            def keywordParts(self):
                return {u'foo': u'bar', u'baz': u'quux'}

            def uniqueIdentifier(self):
                return 'indexable'

            def documentType(self):
                return 'the indexable type'

            def sortKey(self):
                return 'foo'

            def textParts(self):
                return [u'my', u'text']

        indexable = Indexable()
        writer = self.openWriteIndex()
        writer.add(indexable)
        writer.close()

        reader = self.openReadIndex()
        (wrapper,) = reader.search(u'text')

        self.assertEquals(wrapper.keywordParts, indexable.keywordParts())
        self.assertEquals(wrapper.uniqueIdentifier, indexable.uniqueIdentifier())
        self.assertEquals(wrapper.documentType, indexable.documentType())
        self.assertEquals(wrapper.sortKey, indexable.sortKey())


class PyLuceneCorruptionRecoveryTestCase(PyLuceneTestsMixin, CorruptionRecoveryMixin, unittest.TestCase):
    def corruptIndex(self):
        """
        Cause a PyLucene index to appear corrupted.
        """
        for ch in self.store.newFilePath(self.path).children():
            ch.setContent('hello, world')


    def testFailureDetectionFromWriter(self):
        """
        Fulltext indexes are good at two things: soaking up I/O bandwidth and
        corrupting themselves.  For the latter case, we need to be able to
        detect the condition before we can make any response to it.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='10',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.close()
        self.corruptIndex()
        self.assertRaises(fulltext.IndexCorrupt, self.openWriteIndex)
        self.assertRaises(fulltext.IndexCorrupt, self.openReadIndex)


    def testFailureDetectionFromReader(self):
        """
        Like testFailureDetectionFromWriter, but opens a reader after
        corrupting the index and asserts that it also raises the appropriate
        exception.
        """
        writer = self.openWriteIndex()
        writer.add(IndexableThing(
                _documentType=u'thing',
                _uniqueIdentifier='10',
                _textParts=[u'apple', u'banana'],
                _keywordParts={}))
        writer.close()
        self.corruptIndex()
        self.assertRaises(fulltext.IndexCorrupt, self.openReadIndex)
        self.assertRaises(fulltext.IndexCorrupt, self.openWriteIndex)



class PyLuceneLockedRecoveryTestCase(PyLuceneTestsMixin, CorruptionRecoveryMixin, unittest.TestCase):
    def setUp(self):
        CorruptionRecoveryMixin.setUp(self)
        self.corruptedIndexes = []


    def corruptIndex(self):
        """
        Loosely simulate filesystem state following a SIGSEGV or power
        failure.
        """
        self.corruptedIndexes.append(self.openWriteIndex())



class PyLuceneObjectLifetimeTestCase(unittest.TestCase):
    def test_hitsWrapperClosesIndex(self):
        """
        Test that when L{_PyLuceneHitsWrapper} is GC'd, the index which backs
        its C{Hits} object gets closed.
        """
        class TestIndex(object):
            closed = False
            def close(self):
                self.closed = True

        index = TestIndex()
        wrapper = fulltext._PyLuceneHitsWrapper(index, None)
        self.failIf(index.closed)
        del wrapper
        self.failUnless(index.closed)


class IndexerAPISearchTestsMixin(IndexerTestsMixin):
    """
    Test ISearchProvider search API on indexer objects
    """

    def setUp(self):
        """
        Make a store, an account/substore, an indexer, and call startService()
        on the superstore's IService so the batch process interactions that
        happen in fulltext.py work
        """
        self.dbdir = self.mktemp()
        self.path = u'index'

        superstore = store.Store(self.dbdir)

        loginSystem = LoginSystem(store=superstore)
        installOn(loginSystem, superstore)

        account = loginSystem.addAccount(u'testuser', u'example.com', None)
        substore = account.avatars.open()

        self.store = substore
        self.indexer = self.createIndexer()

        self.svc = IService(superstore)
        self.svc.startService()

        # Make sure the indexer is actually available
        writer = self.openWriteIndex()
        writer.close()

    def tearDown(self):
        """
        Stop the service we started in C{setUp}
        """
        return self.svc.stopService()

    def _indexSomeItems(self):
        writer = self.openWriteIndex()
        for i in xrange(5):
            writer.add(IndexableThing(
                        _documentType=u'thing',
                        _uniqueIdentifier=str(i),
                        _textParts=[u'text'],
                        _keywordParts={}))
        writer.close()

    def testIndexerSearching(self):
        """
        Test calling search() on the indexer item directly
        """
        def gotResult(res):
            self.assertEquals(identifiersFrom(res), range(5))
        self._indexSomeItems()
        return self.indexer.search(u'text').addCallback(gotResult)

    def testIndexerSearchingCount(self):
        """
        Test calling search() on the indexer item directly, with a count arg
        """
        def gotResult(res):
            self.assertEquals(identifiersFrom(res), [0])
        self._indexSomeItems()
        return self.indexer.search(u'text', count=1).addCallback(gotResult)

    def testIndexerSearchingOffset(self):
        """
        Test calling search() on the indexer item directly, with an offset arg
        """
        def gotResult(res):
            self.assertEquals(identifiersFrom(res), [1, 2, 3, 4])
        self._indexSomeItems()
        return self.indexer.search(u'text', offset=1).addCallback(gotResult)

    def testIndexerSearchingCountOffset(self):
        """
        Test calling search() on the indexer item directly, with count & offset args
        """
        def gotResult(res):
            self.assertEquals(identifiersFrom(res), [1, 2, 3])
        self._indexSomeItems()
        return self.indexer.search(u'text', count=3, offset=1)


    def test_DifficultTokens(self):
        """
        Test searching for fragments of phone numbers, email
        addresses, and urls.
        """
        writer = self.openWriteIndex()
        specimens = [u"trevor 718-555-1212", u"bob rjones@moddiv.com",
                     u"atop http://divmod.org/projects/atop"]
        for i, txt in enumerate(specimens):
            writer.add(IndexableThing(
                        _documentType=u'thing',
                        _uniqueIdentifier=str(i),
                        _textParts=[txt],
                        _keywordParts={}))
        writer.close()
        def gotResult(res):
            return identifiersFrom(res)
        def testResults(results):
            self.assertEqual(results, [[0], [1], [2],
                                       [0], [1], [2]])
        return gatherResults(
            [self.indexer.search(u'718').addCallback(gotResult),
             self.indexer.search(u'moddiv').addCallback(gotResult),
             self.indexer.search(u'divmod').addCallback(gotResult),
             self.indexer.search(u'718-555').addCallback(gotResult),
             self.indexer.search(u'rjones@moddiv').addCallback(gotResult),
             self.indexer.search(u'divmod.org').addCallback(gotResult),
             ]
            ).addCallback(testResults)

    def test_unicodeSearch(self):
        return self.indexer.search(u'\N{WHITE SMILING FACE}')



class PyLuceneIndexerAPISearchTestCase(PyLuceneTestsMixin, IndexerAPISearchTestsMixin, unittest.TestCase):
    pass



class HypeIndexerAPISearchTestCase(HypeTestsMixin, IndexerAPISearchTestsMixin, unittest.TestCase):
    skip = "These tests don't actually pass - and I don't even care."



def _hasFTS3():
    s = store.Store()
    try:
        s.createSQL('CREATE VIRTUAL TABLE fts USING fts3')
    except SQLError:
        return False
    else:
        return True



class SQLiteTestsMixin(object):
    """
    Mixin for tests for the SQLite indexer.
    """
    if not _hasFTS3():
        skip = 'No FTS3 support'


    def createIndexer(self):
        """
        Create the SQLite indexer.
        """
        return fulltext.SQLiteIndexer(store=self.store, indexDirectory=self.path)



class SQLiteFulltextTestCase(SQLiteTestsMixin, FulltextTestsMixin, unittest.TestCase):
    """
    Tests for SQLite fulltext indexing.
    """
    def _noKeywords(self):
        raise unittest.SkipTest('Keywords are not implemented')


    testKeywordCombination = _noKeywords
    testKeywordIndexing = _noKeywords
    testKeywordTokenization = _noKeywords
    testKeywordValuesInPhrase = _noKeywords
    test_typeRestriction = _noKeywords



class SQLiteIndexerAPISearchTestCase(SQLiteTestsMixin, IndexerAPISearchTestsMixin, unittest.TestCase):
    """
    Tests for SQLite indexing through the indexer service.
    """
    def test_DifficultTokens(self):
        raise unittest.SkipTest("SQLite tokenizer can't handle all of these")



# -*- test-case-name: xmantissa.test.test_ampserver -*-
# Copyright (c) 2008 Divmod.  See LICENSE for details.

"""
This module provides an extensible L{AMP<twisted.protocols.amp>} server for
Mantissa.  The server supports authentication and allows interaction with one
or more L{IBoxReceiver} implementations.  L{IBoxReceiverFactory} powerups are
used to get L{IBoxReceiver} providers by name, routing AMP boxes for multiple
receivers over a single AMP connection.
"""

from zope.interface import implements

from twisted.internet.protocol import ServerFactory
from twisted.internet import reactor
from twisted.cred.portal import Portal
from twisted.protocols.amp import IBoxReceiver, Unicode
from twisted.protocols.amp import BoxDispatcher, CommandLocator, Command
from twisted.python.randbytes import secureRandom

from epsilon.ampauth import CredReceiver, OneTimePadChecker
from epsilon.amprouter import Router

from axiom.iaxiom import IPowerupIndirector
from axiom.item import Item
from axiom.attributes import integer, inmemory
from axiom.dependency import dependsOn
from axiom.userbase import LoginSystem, getLoginMethods
from axiom.upgrade import registerAttributeCopyingUpgrader

from xmantissa.ixmantissa import (
    IProtocolFactoryFactory, IBoxReceiverFactory, IOneTimePadGenerator)

__metaclass__ = type


class AMPConfiguration(Item):
    """
    Configuration object for a Mantissa AMP server.

    @ivar ONE_TIME_PAD_DURATION: The duration of each one-time pad, in
        seconds.
    @type ONE_TIME_PAD_DURATION: C{int}
    """
    powerupInterfaces = (IProtocolFactoryFactory, IOneTimePadGenerator)
    implements(*powerupInterfaces)

    schemaVersion = 2

    loginSystem = dependsOn(LoginSystem)
    _oneTimePads = inmemory()

    ONE_TIME_PAD_DURATION = 60 * 2
    callLater = staticmethod(reactor.callLater)

    def activate(self):
        """
        Initialize L{_oneTimePads}
        """
        self._oneTimePads = {}


    # IOneTimePadGenerator
    def generateOneTimePad(self, userStore):
        """
        Generate a pad which can be used to authenticate via AMP.  This pad
        will expire in L{ONE_TIME_PAD_DURATION} seconds.
        """
        pad = secureRandom(16).encode('hex')
        self._oneTimePads[pad] = userStore.idInParent
        def expirePad():
            self._oneTimePads.pop(pad, None)
        self.callLater(self.ONE_TIME_PAD_DURATION, expirePad)
        return pad


    # IProtocolFactoryFactory
    def getFactory(self):
        """
        Return a server factory which creates AMP protocol instances.
        """
        factory = ServerFactory()
        def protocol():
            proto = CredReceiver()
            proto.portal = Portal(
                self.loginSystem,
                [self.loginSystem,
                 OneTimePadChecker(self._oneTimePads)])
            return proto
        factory.protocol = protocol
        return factory



registerAttributeCopyingUpgrader(
    AMPConfiguration, 1, 2,
    postCopy=lambda new: new.store.powerUp(new, IOneTimePadGenerator))



class ProtocolUnknown(Exception):
    """
    An attempt was made to establish a connection over an AMP route via the
    L{Connect} command to a protocol name for which no factory could be
    located.
    """



class Connect(Command):
    """
    Command to establish a new route to an L{IBoxReceiver} as specified by a
    protocol name.
    """
    arguments = [('origin', Unicode()), ('protocol', Unicode())]
    response = [('route', Unicode())]

    errors = {ProtocolUnknown: 'PROTOCOL_UNKNOWN'}



class _RouteConnector(BoxDispatcher, CommandLocator):
    """
    The actual L{IBoxReceiver} implementation supplied by L{AMPRouter} to be
    used as the avatar.  There is one L{_RouteConnector} instance per
    connection.

    @ivar reactor: An L{IReactorTime} provider which will be used to schedule
        the actual route connection.  This is a workaround for a missing AMP
        API: the ability to indicate a response without returning a value from
        a responder.  The application L{IBoxReceiver} being associated with a
        new route cannot be allowed to send any boxes before the I{Connect}
        response box is sent.  The simplest way to do this is to schedule a
        timed call to do the route connection so that it happens after the
        responder returns.

    @ivar store: The L{Store} which contained the L{AMPRouter} which created
        this object and which will be used to find L{IBoxReceiverFactory}
        powerups.

    @ivar router: The L{Router} which will be used to create new routes.
    """
    def __init__(self, reactor, store, router):
        BoxDispatcher.__init__(self, self)
        self.reactor = reactor
        self.store = store
        self.router = router


    @Connect.responder
    def accept(self, origin, protocol):
        """
        Create a new route attached to a L{IBoxReceiver} created by the
        L{IBoxReceiverFactory} with the indicated protocol.

        @type origin: C{unicode}
        @param origin: The identifier of a route on the peer which will be
            associated with this connection.  Boxes sent back by the protocol
            which is created in this call will be sent back to this route.

        @type protocol: C{unicode}
        @param protocol: The name of the protocol to which to establish a
            connection.

        @raise ProtocolUnknown: If no factory can be found for the named
            protocol.

        @return: A newly created C{unicode} route identifier for this
            connection (as the value of a C{dict} with a C{'route'} key).
        """
        for factory in self.store.powerupsFor(IBoxReceiverFactory):
            # XXX What if there's a duplicate somewhere?
            if factory.protocol == protocol:
                receiver = factory.getBoxReceiver()
                route = self.router.bindRoute(receiver)
                # This might be better implemented using a hook on the box.
                # See Twisted ticket #3479.
                self.reactor.callLater(0, route.connectTo, origin)
                return {'route': route.localRouteName}
        raise ProtocolUnknown()



class AMPAvatar(Item):
    """
    An L{IBoxReceiver} avatar which multiplexes AMP boxes for other receiver
    powerups on this item's store.
    """
    powerupInterfaces = (IBoxReceiver,)
    implements(IPowerupIndirector)

    garbage = integer(
        doc="""
        This class is stateless.  This attribute satisfies the Axiom
        requirement that L{Item} subclasses have at least one attribute.
        """)


    def connectorFactory(self, router):
        """
        Create the default receiver to use with the L{Router} returned
        by C{indirect}.
        """
        return _RouteConnector(reactor, self.store, router)


    def indirect(self, interface):
        """
        Create a L{Router} to handle AMP boxes received over an AMP connection.
        """
        if interface is IBoxReceiver:
            router = Router()
            connector = self.connectorFactory(router)
            router.bindRoute(connector, None).connectTo(None)
            return router
        raise NotImplementedError()



def connectRoute(amp, router, receiver, protocol):
    """
    Connect the given receiver to a new box receiver for the given
    protocol.

    After connecting this router to an AMP server, use this method
    similarly to how you would use C{reactor.connectTCP} to establish a new
    connection to an HTTP, SMTP, or IRC server.

    @param receiver: An L{IBoxReceiver} which will be started when a route
        to a receiver for the given protocol is found.

    @param protocol: The name of a protocol which the AMP peer to which
        this router is connected has an L{IBoxReceiverFactory}.

    @return: A L{Deferred} which fires with C{receiver} when the route is
        established.
    """
    route = router.bindRoute(receiver)
    d = amp.callRemote(
        Connect,
        origin=route.localRouteName,
        protocol=protocol)
    def cbGotRoute(result):
        route.connectTo(result['route'])
        return receiver
    d.addCallback(cbGotRoute)
    return d



class EchoFactory(Item):
    """
    Box receiver factory for an AMP protocol which just echoes AMP boxes back
    to the sender.  This is primarily useful as an example of a box receiver
    factory and as a way to test whether it is possible to connect to protocols
    over a Mantissa AMP server (similar to VoIP echo tests).
    """
    powerupInterfaces = (IBoxReceiverFactory,)
    implements(*powerupInterfaces)

    protocol = u"http://divmod.org/ns/echo"

    _garbage = integer(
        doc="""
        meaningless attribute, only here to satisfy Axiom requirement for at
        least one attribute.
        """)

    def getBoxReceiver(self):
        return EchoReceiver()



class EchoReceiver:
    """
    An AMP box echoer.
    """
    implements(IBoxReceiver)

    def startReceivingBoxes(self, sender):
        self.sender = sender


    def ampBoxReceived(self, box):
        self.sender.sendBox(box)


    def stopReceivingBoxes(self, reason):
        pass



__all__ = [
    'ProtocolUnknown', 'RouteNotConnected',

    'Connect', 'connectRoute',

    'AMPConfiguration', 'AMPAvatar',

    'Router', 'MantissaRouter',

    'EchoFactory', 'EchoReceiver']



# -*- test-case-name: xmantissa.test.test_cachejs -*-
"""
This module implements a strategy for allowing the browser to cache
JavaScript modules served by Athena.  It's not entirely standalone, as it
requires some cooperation from L{xmantissa.website}, specifically
L{xmantissa.website.MantissaLivePage}.
"""

import sha

from zope.interface import implements

from twisted.python.filepath import FilePath

from nevow.inevow import IResource
from nevow import athena
from nevow import static
from nevow.rend import NotFound, FourOhFour


class CachedJSModule(object):
    """
    Various bits of cached information about a JavaScript module.

    @ivar moduleName: A JavaScript module name, e.g. 'Foo.Bar'.
    @type moduleName: C{str}.

    @ivar filePath: The path to the module on the filesystem.
    @type filePath: L{FilePath}.

    @ivar lastModified: The mtime of L{filePath}, as a POSIX timestamp.
    @type lastModified: C{int}.
    """

    def __init__(self, moduleName, filePath):
        """
        Create a CachedJSModule.
        """
        self.moduleName = moduleName
        self.filePath = filePath
        self.lastModified = 0
        self.maybeUpdate()


    def wasModified(self):
        """
        Check to see if this module has been modified on disk since the last
        time it was cached.

        @return: True if it has been modified, False if not.
        """
        self.filePath.restat()
        mtime = self.filePath.getmtime()
        if mtime >= self.lastModified:
            return True
        else:
            return False


    def maybeUpdate(self):
        """
        Check this cache entry and update it if any filesystem information has
        changed.
        """
        if self.wasModified():
            self.lastModified = self.filePath.getmtime()
            self.fileContents = self.filePath.getContent()
            self.hashValue = sha.new(self.fileContents).hexdigest()



class HashedJSModuleProvider(object):
    """
    An Athena module-serving resource which handles hashed names instead of
    regular module names.

    @ivar moduleCache: a map of JS module names to CachedJSModule objects,
    representing the filesystem locations and contents of the modules.

    @ivar depsMemo: A memo of module dependencies.
    @type depsMemo: C{dict} of C{module name: dependent modules}
    """
    implements(IResource)

    def __init__(self):
        """
        Create a HashedJSModuleProvider.
        """
        self.moduleCache = {}
        self.depsMemo = {}


    def getModule(self, moduleName):
        """
        Retrieve a JavaScript module cache from the file path cache.

        @returns: Module cache for the named module.
        @rtype: L{CachedJSModule}
        """
        if moduleName not in self.moduleCache:
            modulePath = FilePath(
                athena.jsDeps.getModuleForName(moduleName)._cache.path)
            cachedModule = self.moduleCache[moduleName] = CachedJSModule(
                moduleName, modulePath)
        else:
            cachedModule = self.moduleCache[moduleName]
        return cachedModule


    # IResource
    def locateChild(self, ctx, segments):
        """
        Retrieve an L{inevow.IResource} to render the contents of the given
        module.
        """
        if len(segments) != 2:
            return NotFound
        hashCode, moduleName = segments
        cachedModule = self.getModule(moduleName)
        return static.Data(
            cachedModule.fileContents,
            'text/javascript', expires=(60 * 60 * 24 * 365 * 5)), []


    def renderHTTP(self, ctx):
        """
        There is no index of javascript modules; this resource is not
        renderable.
        """
        return FourOhFour()



theHashModuleProvider = HashedJSModuleProvider()

__all__ = ['HashedJSModuleProvider',
           'theHashModuleProvider']



# -*- test-case-name: xmantissa.test.test_publicweb -*-
# Copyright 2008 Divmod, Inc. See LICENSE file for details
"""
This module contains code for the publicly-visible areas of a Mantissa
server's web interface.
"""

from warnings import warn

from zope.interface import implements

from twisted.internet import defer

from nevow.inevow import IRequest, IResource
from nevow import rend, tags, inevow
from nevow.url import URL

from axiom.iaxiom import IPowerupIndirector
from axiom import item, attributes, upgrade, userbase
from axiom.dependency import dependsOn, requiresFromSite

from xmantissa import ixmantissa, website, offering
from xmantissa._webutil import (MantissaViewHelper, SiteRootMixin,
                                WebViewerHelper)
from xmantissa.webtheme import (ThemedDocumentFactory, getInstalledThemes,
                                SiteTemplateResolver)
from xmantissa.ixmantissa import (
    IStaticShellContent, ISiteRootPlugin, IMantissaSite, IWebViewer,
    INavigableFragment)
from xmantissa.webnav import startMenu, settingsLink, applicationNavigation
from xmantissa.websharing import UserIndexPage, SharingIndex, getDefaultShareID
from xmantissa.sharing import getEveryoneRole, NoSuchShare


def getLoader(*a, **kw):
    """
    Deprecated.  Don't use this.
    """
    warn("xmantissa.publicweb.getLoader is deprecated, use "
         "PrivateApplication.getDocFactory or SiteTemplateResolver."
         "getDocFactory.", category=DeprecationWarning, stacklevel=2)
    from xmantissa.webtheme import getLoader
    return getLoader(*a, **kw)



def renderShortUsername(ctx, username):
    """
    Render a potentially shortened version of the user's login identifier,
    depending on how the user is viewing it.  For example, if bob@example.com
    is viewing http://example.com/private/, then render 'bob'.  If bob instead
    signed up with only his email address (bob@hotmail.com), and is viewing a
    page at example.com, then render the full address, 'bob@hotmail.com'.

    @param ctx: a L{WovenContext} which has remembered IRequest.

    @param username: a string of the form localpart@domain.

    @return: a L{Tag}, the given context's tag, with the appropriate username
    appended to it.
    """
    if username is None:
        return ''
    req = inevow.IRequest(ctx)
    localpart, domain = username.split('@')
    host = req.getHeader('Host').split(':')[0]
    if host == domain or host.endswith("." + domain):
        username = localpart
    return ctx.tag[username]



class PublicWeb(item.Item, website.PrefixURLMixin):
    """
    Fixture for site-wide public-facing content.

    I implement ISiteRootPlugin and use PrefixURLMixin; see the documentation
    for each of those for a detailed explanation of my usage.

    I wrap a L{websharing.SharingIndex} around an app store. I am installed in
    an app store when it is created.
    """
    implements(ISiteRootPlugin,
               ixmantissa.ISessionlessSiteRootPlugin)

    typeName = 'mantissa_public_web'
    schemaVersion = 3

    prefixURL = attributes.text(
        doc="""
        The prefix of the URL where objects represented by this fixture will
        appear.  For the front page this is u'', for other pages it is their
        respective URLs.
        """, allowNone=False)

    application = attributes.reference(
        doc="""
        A L{SubStore} for an application store.
        """,
        allowNone=False)

    installedOn = attributes.reference(
        doc="""
        """)

    sessioned = attributes.boolean(
        doc="""
        Will this resource be provided to clients with a session?  Defaults to
        False.
        """,
        default=False)

    sessionless = attributes.boolean(
        doc="""
        Will this resource be provided without a session to clients without a
        session?  Defaults to False.
        """,
        default=False)


    def createResource(self):
        """
        When invoked by L{PrefixURLMixin}, return a L{websharing.SharingIndex}
        for my application.
        """
        pp = ixmantissa.IPublicPage(self.application, None)
        if pp is not None:
            warn(
            "Use the sharing system to provide public pages, not IPublicPage",
            category=DeprecationWarning,
            stacklevel=2)
            return pp.getResource()
        return SharingIndex(self.application.open())


def upgradePublicWeb1To2(oldWeb):
    newWeb = oldWeb.upgradeVersion(
        'mantissa_public_web', 1, 2,
        prefixURL=oldWeb.prefixURL,
        application=oldWeb.application,
        installedOn=oldWeb.installedOn)
    newWeb.installedOn.powerUp(newWeb, ixmantissa.ICustomizablePublicPage)
    return newWeb
upgrade.registerUpgrader(upgradePublicWeb1To2, 'mantissa_public_web', 1, 2)

def upgradePublicWeb2To3(oldWeb):
    newWeb = oldWeb.upgradeVersion(
        'mantissa_public_web', 2, 3,
        prefixURL=oldWeb.prefixURL,
        application=oldWeb.application,
        installedOn=oldWeb.installedOn,
        # There was only one PublicWeb before, and it definitely
        # wanted to be sessioned.
        sessioned=True)
    newWeb.installedOn.powerDown(newWeb, ixmantissa.ICustomizablePublicPage)
    other = newWeb.installedOn
    newWeb.installedOn = None
    newWeb.installOn(other)
    return newWeb
upgrade.registerUpgrader(upgradePublicWeb2To3, 'mantissa_public_web', 2, 3)



class _AnonymousWebViewer(WebViewerHelper):
    """
    An implementation of L{IWebViewer} for anonymous users.

    @ivar _siteStore: A site store that contains an L{AnonymousSite} and
        L{SiteConfiguration}.

    @ivar _getDocFactory: the L{SiteTemplateResolver.getDocFactory} method that
        will resolve themes for my site store.
    """
    implements(IWebViewer)

    def __init__(self, siteStore):
        """
        Create an L{_AnonymousWebViewer} for browsing a given site store.
        """
        WebViewerHelper.__init__(
            self,
            SiteTemplateResolver(siteStore).getDocFactory,
            lambda : getInstalledThemes(siteStore))
        self._siteStore = siteStore


    # IWebViewer
    def roleIn(self, userStore):
        """
        Return only the 'everyone' role in the given user- or app-store, since
        the user represented by this object is anonymous.
        """
        return getEveryoneRole(userStore)


    # Complete WebViewerHelper implementation
    def _wrapNavFrag(self, frag, useAthena):
        """
        Wrap the given L{INavigableFragment} in the appropriate type of
        L{_PublicPageMixin}.
        """
        if useAthena:
            return PublicAthenaLivePage(self._siteStore, frag)
        else:
            return PublicPage(None, self._siteStore, frag, None, None)



class _CustomizingResource(object):
    """
    _CustomizingResource is a wrapping resource used to implement
    CustomizedPublicPage.

        There is an implementation assumption here, which is that the top
        _CustomizingResource is always at "/", and locateChild will always be
        invoked at least once.  If this doesn't hold, this render method might
        be invoked on the top level _CustomizingResource, which would cause it
        to be rendered without customization.  If you're going to use this
        class directly for some reason, please keep this in mind.
    """
    implements(inevow.IResource)

    def __repr__(self):
        return '<Customizing Resource %r: %r>' % (self.forWho, self.currentResource)


    def __init__(self, topResource, forWho):
        """
        Create a _CustomizingResource.

        @param topResource: an L{inevow.IResource} provider, who may also
        provide L{ixmantissa.ICustomizable} if it wishes to be customized.

        @param forWho: the external ID of the currently logged-in user.
        @type forWho: unicode
        """
        self.currentResource = topResource
        self.forWho = forWho


    def locateChild(self, ctx, segments):
        """
        Return a Deferred which will fire with the customized version of the
        resource being located.
        """
        D = defer.maybeDeferred(
            self.currentResource.locateChild, ctx, segments)

        def finishLocating((nextRes, nextPath)):
            custom = ixmantissa.ICustomizable(nextRes, None)
            if custom is not None:
                return (custom.customizeFor(self.forWho), nextPath)
            self.currentResource = nextRes
            if nextRes is None:
                return (nextRes, nextPath)
            return (_CustomizingResource(nextRes, self.forWho), nextPath)

        return D.addCallback(finishLocating)


    def renderHTTP(self, ctx):
        """
        Render the resource I was provided at construction time.
        """
        if self.currentResource is None:
            return rend.FourOhFour()
        return self.currentResource # nevow will automatically adapt to
                                    # IResource and call rendering methods.


class CustomizedPublicPage(item.Item):
    """
    L{CustomizedPublicPage} is what allows logged-in users to see dynamic
    resources present in the site store.  Although static resources (under
    http://your-domain.example.com/static) are available to everyone, any user
    who should be able to see content such as http://yoursite/users/some-user/
    when they are logged in must have this installed.
    """
    implements(ISiteRootPlugin)

    typeName = 'mantissa_public_customized'
    schemaVersion = 2

    installedOn = attributes.reference(
        doc="""
        The Avatar for which this item will attempt to retrieve a customized
        page.
        """)

    powerupInterfaces = [(ISiteRootPlugin, -257)]

    publicSiteRoot = requiresFromSite(IMantissaSite, lambda ignored: None)

    def produceResource(self, request, segments, webViewer):
        """
        Produce a resource that traverses site-wide content, passing down the
        given webViewer.  This delegates to the site store's
        L{IMantissaSite} adapter, to avoid a conflict with the
        L{ISiteRootPlugin} interface.

        This method will typically be given an L{_AuthenticatedWebViewer}, which
        can build an appropriate resource for an authenticated shell page,
        whereas the site store's L{IWebViewer} adapter would show an anonymous
        page.

        The result of this method will be a L{_CustomizingResource}, to provide
        support for resources which may provide L{ICustomizable}.  Note that
        Mantissa itself no longer implements L{ICustomizable} anywhere, though.
        All application code should phase out inspecting the string passed to
        ICustomizable in favor of getting more structured information from the
        L{IWebViewer}.  However, it has not been deprecated yet because
        the interface which allows application code to easily access the
        L{IWebViewer} from view code has not yet been developed; it is
        forthcoming.

        See ticket #2707 for progress on this.
        """
        mantissaSite = self.publicSiteRoot
        if mantissaSite is not None:
            for resource, domain in userbase.getAccountNames(self.store):
                username = '%s@%s' % (resource, domain)
                break
            else:
                username = None
            bottomResource, newSegments = mantissaSite.siteProduceResource(
                request, segments, webViewer)
            return (_CustomizingResource(bottomResource, username), newSegments)
        return None



def customizedPublicPage1To2(oldPage):
    newPage = oldPage.upgradeVersion(
        'mantissa_public_customized', 1, 2,
        installedOn=oldPage.installedOn)
    newPage.installedOn.powerDown(newPage, ISiteRootPlugin)
    newPage.installedOn.powerUp(newPage, ISiteRootPlugin, -257)
    return newPage
upgrade.registerUpgrader(customizedPublicPage1To2, 'mantissa_public_customized', 1, 2)



class _PublicPageMixin(MantissaViewHelper):
    """
    Mixin for use by C{Page} or C{LivePage} subclasses that are visible to
    unauthenticated clients.

    @ivar store: The site store.
    """
    fragment = None
    username = None


    def _getViewerPrivateApplication(self):
        """
        Get the L{PrivateApplication} object for the logged-in user who is
        viewing this resource, as indicated by its C{username} attribute.

        This is highly problematic because it precludes the possibility of
        separating the stores of the viewer and the viewee into separate
        processes, and it is only here until we can get rid of it.  The reason
        it remains is that some application code still imports things which
        subclass L{PublicAthenaLivePage} and L{PublicPage} and uses them with
        usernames specified.  See ticket #2702 for progress on this goal.

        However, Mantissa itself will no longer set this class's username
        attribute to anything other than None, because authenticated users'
        pages will be generated using
        L{xmantissa.webapp._AuthenticatedWebViewer}.  This method is used only
        to render content in the shell template, and those classes have a direct
        reference to the requisite object.

        @rtype: L{PrivateApplication}
        """
        ls = self.store.findUnique(userbase.LoginSystem)
        substore = ls.accountByAddress(*self.username.split('@')).avatars.open()
        from xmantissa.webapp import PrivateApplication
        return substore.findUnique(PrivateApplication)


    def render_authenticateLinks(self, ctx, data):
        """
        For unauthenticated users, add login and signup links to the given tag.
        For authenticated users, remove the given tag from the output.

        When necessary, the I{signup-link} pattern will be loaded from the tag.
        Each copy of it will have I{prompt} and I{url} slots filled.  The list
        of copies will be added as children of the tag.
        """
        if self.username is not None:
            return ''
        # there is a circular import here which should probably be avoidable,
        # since we don't actually need signup links on the signup page.  on the
        # other hand, maybe we want to eventually put those there for
        # consistency.  for now, this import is easiest, and although it's a
        # "friend" API, which I dislike, it doesn't seem to cause any real
        # problems...  -glyph
        from xmantissa.signup import _getPublicSignupInfo

        IQ = inevow.IQ(ctx.tag)
        signupPattern = IQ.patternGenerator('signup-link')

        signups = []
        for (prompt, url) in _getPublicSignupInfo(self.store):
            signups.append(signupPattern.fillSlots(
                    'prompt', prompt).fillSlots(
                    'url', url))

        return ctx.tag[signups]


    def render_startmenu(self, ctx, data):
        """
        For authenticated users, add the start-menu style navigation to the
        given tag.  For unauthenticated users, remove the given tag from the
        output.

        @see L{xmantissa.webnav.startMenu}
        """
        if self.username is None:
            return ''
        translator = self._getViewerPrivateApplication()
        pageComponents = translator.getPageComponents()
        return startMenu(translator, pageComponents.navigation, ctx.tag)


    def render_settingsLink(self, ctx, data):
        """
        For authenticated users, add the URL of the settings page to the given
        tag.  For unauthenticated users, remove the given tag from the output.
        """
        if self.username is None:
            return ''
        translator = self._getViewerPrivateApplication()
        return settingsLink(
            translator,
            translator.getPageComponents().settings,
            ctx.tag)


    def render_applicationNavigation(self, ctx, data):
        """
        For authenticated users, add primary application navigation to the
        given tag.  For unauthenticated users, remove the given tag from the
        output.

        @see L{xmantissa.webnav.applicationNavigation}
        """
        if self.username is None:
            return ''
        translator = self._getViewerPrivateApplication()
        return applicationNavigation(
            ctx,
            translator,
            translator.getPageComponents().navigation)


    def render_search(self, ctx, data):
        """
        Render some UI for performing searches, if we know about a search
        aggregator.
        """
        if self.username is None:
            return ''
        translator = self._getViewerPrivateApplication()
        searchAggregator = translator.getPageComponents().searchAggregator
        if searchAggregator is None or not searchAggregator.providers():
            return ''
        return ctx.tag.fillSlots(
            'form-action', translator.linkTo(searchAggregator.storeID))


    def render_username(self, ctx, data):
        return renderShortUsername(ctx, self.username)


    def render_logout(self, ctx, data):
        if self.username is None:
            return ''
        return ctx.tag


    def render_title(self, ctx, data):
        """
        Return the current context tag containing C{self.fragment}'s C{title}
        attribute, or "Divmod".
        """
        return ctx.tag[getattr(self.fragment, 'title', 'Divmod')]


    def render_rootURL(self, ctx, data):
        """
        Add the WebSite's root URL as a child of the given tag.
        """
        return ctx.tag[
            ixmantissa.ISiteURLGenerator(self.store).rootURL(IRequest(ctx))]


    def render_header(self, ctx, data):
        """
        Render any required static content in the header, from the C{staticContent}
        attribute of this page.
        """
        if self.staticContent is None:
            return ctx.tag

        header = self.staticContent.getHeader()
        if header is not None:
            return ctx.tag[header]
        else:
            return ctx.tag


    def render_footer(self, ctx, data):
        """
        Render any required static content in the footer, from the C{staticContent}
        attribute of this page.
        """
        if self.staticContent is None:
            return ctx.tag

        header = self.staticContent.getFooter()
        if header is not None:
            return ctx.tag[header]
        else:
            return ctx.tag


    def render_urchin(self, ctx, data):
        """
        Render the code for recording Google Analytics statistics, if so
        configured.
        """
        key = website.APIKey.getKeyForAPI(self.store, website.APIKey.URCHIN)
        if key is None:
            return ''
        return ctx.tag.fillSlots('urchin-key', key.apiKey)


    def render_content(self, ctx, data):
        """
        This renderer, which is used for the visual bulk of the page, provides
        self.fragment renderer.
        """
        return ctx.tag[self.fragment]


    def getHeadContent(self, req):
        """
        Retrieve a list of header content from all installed themes on the site
        store.
        """
        site = ixmantissa.ISiteURLGenerator(self.store)
        for t in getInstalledThemes(self.store):
            yield t.head(req, site)


    def render_head(self, ctx, data):
        """
        This renderer calculates content for the <head> tag by concatenating the
        values from L{getHeadContent} and the overridden L{head} method.
        """
        req = inevow.IRequest(ctx)
        more = getattr(self.fragment, 'head', None)
        if more is not None:
            fragmentHead = more()
        else:
            fragmentHead = None
        return ctx.tag[filter(None, list(self.getHeadContent(req)) +
                              [fragmentHead])]



class PublicPage(_PublicPageMixin, rend.Page):
    """
    PublicPage is a utility superclass for implementing static pages which have
    theme support and authentication trimmings.
    """
    docFactory = ThemedDocumentFactory('shell', 'templateResolver')

    def __init__(self, original, store, fragment, staticContent, forUser,
                 templateResolver=None):
        """
        Create a public page.

        @param original: any object

        @param store: a site store containing a L{WebSite}.
        @type store: L{axiom.store.Store}.

        @param fragment: a L{rend.Fragment} to display in the content area of
        the page.

        @param staticContent: some stan, to include in the header of the page.

        @param forUser: a string, the external ID of a user to customize for.

        @param templateResolver: a template resolver instance that will return
        the appropriate doc factory.
        """
        if templateResolver is None:
            templateResolver = ixmantissa.ITemplateNameResolver(store)
        super(PublicPage, self).__init__(original)
        self.store = store
        self.fragment = fragment
        self.staticContent = staticContent
        self.templateResolver = templateResolver
        if forUser is not None:
            assert isinstance(forUser, unicode), forUser
        self.username = forUser



class _OfferingsFragment(rend.Fragment):
    """
    This fragment provides the list of installed offerings as a data generator.
    This is used to display the list of app stores on the default front page.

    @ivar templateResolver: An L{ITemplateNameResolver} which will be used
        to load the document factory.
    """
    implements(INavigableFragment)
    docFactory = ThemedDocumentFactory('front-page', 'templateResolver')

    def __init__(self, original, templateResolver=None):
        """
        Create an _OfferingsFragment with an item from a site store.

        @param original: a L{FrontPage} item.

        @param templateResolver: An L{ITemplateNameResolver} from which to
            load the document factory.  If not specified, the Store of
            C{original} will be adapted to L{ITemplateNameResolver} and used
            for this purpose.  It is recommended that you pass a value for
            this parameter.
        """
        if templateResolver is None:
            templateResolver = ixmantissa.ITemplateNameResolver(original.store)
        self.templateResolver = templateResolver
        super(_OfferingsFragment, self).__init__(original)


    def data_offerings(self, ctx, data):
        """
        Generate a list of installed offerings.

        @return: a generator of dictionaries mapping 'name' to the name of an
        offering installed on the store.
        """
        for io in self.original.store.query(offering.InstalledOffering):
            pp = ixmantissa.IPublicPage(io.application, None)
            if pp is not None and getattr(pp, 'index', True):
                warn("Use the sharing system to provide public pages,"
                     " not IPublicPage",
                     category=DeprecationWarning,
                     stacklevel=2)
                yield {'name': io.offeringName}
            else:
                s = io.application.open()
                try:
                    pp = getEveryoneRole(s).getShare(getDefaultShareID(s))
                    yield {'name': io.offeringName}
                except NoSuchShare:
                    continue



class _PublicFrontPage(object):
    """
    This is the implementation of the default Mantissa front page.  It renders
    a list of offering names, displays the user's name, and lists signup
    mechanisms.  It also provides various top-level URLs.
    """
    implements(IResource)

    def __init__(self, frontPageItem, webViewer):
        """
        Create a _PublicFrontPage.

        @param frontPageItem: a L{FrontPage} item, which we use primarily to
        get at a Store.

        @param webViewer: an L{IWebViewer} that represents the
        user viewing this front page.
        """
        self.frontPageItem = frontPageItem
        self.webViewer = webViewer


    def locateChild(self, ctx, segments):
        """
        Look up children in the normal manner, but then customize them for the
        authenticated user if they support the L{ICustomizable} interface.  If
        the user is attempting to access a private URL, redirect them.
        """
        result = self._getAppStoreResource(ctx, segments[0])
        if result is not None:
            child, segments = result, segments[1:]
            return child, segments

        if segments[0] == '':
            result = self.child_(ctx)
            if result is not None:
                child, segments = result, segments[1:]
                return child, segments

        # If the user is trying to access /private/*, then his session has
        # expired or he is otherwise not logged in. Redirect him to /login,
        # preserving the URL segments, rather than giving him an obscure 404.
        if segments[0] == 'private':
            u = URL.fromContext(ctx).click('/').child('login')
            for seg in segments:
                u = u.child(seg)
            return u, ()

        return rend.NotFound


    def _getAppStoreResource(self, ctx, name):
        """
        Customize child lookup such that all installed offerings on the site
        store that this page is viewing are given an opportunity to display
        their own page.
        """
        offer = self.frontPageItem.store.findFirst(
            offering.InstalledOffering,
            offering.InstalledOffering.offeringName == unicode(name, 'ascii'))
        if offer is not None:
            pp = ixmantissa.IPublicPage(offer.application, None)
            if pp is not None:
                warn("Use the sharing system to provide public pages,"
                     " not IPublicPage",
                     category=DeprecationWarning,
                     stacklevel=2)
                return pp.getResource()
            return SharingIndex(offer.application.open(),
                                self.webViewer)
        return None


    def child_(self, ctx):
        """
        If the root resource is requested, return the primary
        application's front page, if a primary application has been
        chosen.  Otherwise return 'self', since this page can render a
        simple index.
        """
        if self.frontPageItem.defaultApplication is None:
            return self.webViewer.wrapModel(
                _OfferingsFragment(self.frontPageItem))
        else:
            return SharingIndex(self.frontPageItem.defaultApplication.open(),
                                self.webViewer).locateChild(ctx, [''])[0]



class LoginPage(PublicPage):
    """
    This is the page which presents a 'login' dialog to the user, at "/login".

    This does not perform the actual login, nevow.guard does that, at the URL
    /__login__; this resource merely provides the entry field and redirection
    logic.
    """

    # Try to get SSL if possible.  See xmantissa.web.SecuringWrapper and
    # xmantissa.web.SiteConfiguration.getFactory.  This should really be
    # indicated in some other way.  See #2525 -exarkun
    needsSecure = True

    def __init__(self, store, segments=(), arguments=None,
                 templateResolver=None):
        """
        Create a login page.

        @param store: a site store containing a L{WebSite}.
        @type store: L{axiom.store.Store}.

        @param segments: a list of strings.  For example, if you hit
        /login/private/stuff, you want to log in to /private/stuff, and the
        resulting LoginPage will have the segments of ['private', 'stuff']

        @param arguments: A dictionary mapping query argument names to lists of
        values for those arguments (see IRequest.args).

        @param templateResolver: a template resolver instance that will return
        the appropriate doc factory.
        """
        if templateResolver is None:
            templateResolver = ixmantissa.ITemplateNameResolver(store)
        PublicPage.__init__(self, None, store,
                            templateResolver.getDocFactory('login'),
                            IStaticShellContent(store, None),
                            None, templateResolver)
        self.segments = segments
        if arguments is None:
            arguments = {}
        self.arguments = arguments


    def beforeRender(self, ctx):
        """
        Before rendering this page, identify the correct URL for the login to post
        to, and the error message to display (if any), and fill the 'login
        action' and 'error' slots in the template accordingly.
        """
        generator = ixmantissa.ISiteURLGenerator(self.store)
        url = generator.rootURL(IRequest(ctx))
        url = url.child('__login__')
        for seg in self.segments:
            url = url.child(seg)
        for queryKey, queryValues in self.arguments.iteritems():
            for queryValue in queryValues:
                url = url.add(queryKey, queryValue)

        req = inevow.IRequest(ctx)
        err = req.args.get('login-failure', ('',))[0]

        if 0 < len(err):
            error = inevow.IQ(
                        self.fragment).onePattern(
                                'error').fillSlots('error', err)
        else:
            error = ''

        ctx.fillSlots("login-action", url)
        ctx.fillSlots("error", error)


    def locateChild(self, ctx, segments):
        """
        Return a clone of this page that remembers its segments, so that URLs like
        /login/private/stuff will redirect the user to /private/stuff after
        login has completed.
        """
        arguments = IRequest(ctx).args
        return self.__class__(
            self.store, segments, arguments), ()


    def fromRequest(cls, store, request):
        """
        Return a L{LoginPage} which will present the user with a login prompt.

        @type store: L{Store}
        @param store: A I{site} store.

        @type request: L{nevow.inevow.IRequest}
        @param request: The HTTP request which encountered a need for
            authentication.  This will be effectively re-issued after login
            succeeds.

        @return: A L{LoginPage} and the remaining segments to be processed.
        """
        location = URL.fromRequest(request)
        segments = location.pathList(unquote=True, copy=False)
        segments.append(request.postpath[0])
        return cls(store, segments, request.args)
    fromRequest = classmethod(fromRequest)



class FrontPage(item.Item, website.PrefixURLMixin):
    """
    I am a factory for the dynamic resource L{_PublicFrontPage}.
    """
    implements(ISiteRootPlugin)
    typeName = 'mantissa_front_page'
    schemaVersion = 2

    sessioned = True

    publicViews = attributes.integer(
        doc="""
        The number of times this object has been viewed anonymously.  This
        includes renderings of the front page only.
        """,
        default=0)

    privateViews = attributes.integer(
        doc="""
        The number of times this object has been viewed non-anonymously.  This
        includes renderings of the front page only.
        """,
        default=0)

    prefixURL = attributes.text(
        doc="""
        See L{website.PrefixURLMixin}.
        """,
        default=u'',
        allowNone=False)

    defaultApplication = attributes.reference(
        doc="""
        An application L{SubStore} whose default shared item should be
        displayed on the root web resource. If None, the default index
        of applications will be displayed.
        """,
        allowNone=True)

    def createResourceWith(self, crud):
        """
        Create a L{_PublicFrontPage} resource wrapping this object.
        """
        return _PublicFrontPage(self, crud)

item.declareLegacyItem(
    FrontPage.typeName,
    1,
    dict(publicViews = attributes.integer(),
         privateViews = attributes.integer(),
         prefixURL = attributes.text(allowNone=False)))

upgrade.registerAttributeCopyingUpgrader(FrontPage, 1, 2)



class PublicAthenaLivePage(_PublicPageMixin, website.MantissaLivePage):
    """
    PublicAthenaLivePage is a publicly viewable Athena-enabled page which slots
    a single fragment into the center of the page.
    """
    docFactory = ThemedDocumentFactory('shell', 'templateResolver')
    unsupportedBrowserLoader = ThemedDocumentFactory(
        'athena-unsupported', 'templateResolver')

    fragment = None

    def __init__(self, store, fragment, staticContent=None, forUser=None,
                 templateResolver=None,
                 *a, **kw):
        """
        Create a PublicAthenaLivePage.

        @param store: a site store containing an L{AnonymousSite} and
            L{SiteConfiguration}.

        @type store: L{axiom.store.Store}.

        @param fragment: The L{INavigableFragment} provider which will be
            displayed on this page.

        @param templateResolver: a template resolver instance that will return
            the appropriate doc factory.

        This page draws its HTML from the 'shell' template in the active theme.
        If loaded in a browser that does not support Athena, the page provided
        by the 'athena-unsupported' template will be displayed instead.
        """
        self.store = store
        if templateResolver is None:
            templateResolver = ixmantissa.ITemplateNameResolver(self.store)
        self.templateResolver = templateResolver
        super(PublicAthenaLivePage, self).__init__(ixmantissa.ISiteURLGenerator(self.store), *a, **kw)
        if fragment is not None:
            self.fragment = fragment
            # everybody who calls this has a different idea of what 'fragment'
            # means - let's just be super-lenient for now
            if getattr(fragment, 'setFragmentParent', False):
                fragment.setFragmentParent(self)
            else:
                fragment.page = self
        self.staticContent = staticContent
        self.username = forUser


    def render_head(self, ctx, data):
        """
        Put liveglue content into the header of this page to activate it, but
        otherwise delegate to my parent's renderer for <head>.
        """
        ctx.tag[tags.invisible(render=tags.directive('liveglue'))]
        return _PublicPageMixin.render_head(self, ctx, data)



class PublicNavAthenaLivePage(PublicAthenaLivePage):
    """
    DEPRECATED!  Use PublicAthenaLivePage.

    A L{PublicAthenaLivePage} which optionally includes a menubar and
    navigation if the viewer is authenticated.
    """
    def __init__(self, *a, **kw):
        PublicAthenaLivePage.__init__(self, *a, **kw)
        warn(
            "Use PublicAthenaLivePage instead of PublicNavAthenaLivePage",
            category=DeprecationWarning,
            stacklevel=2)



class AnonymousSite(item.Item, SiteRootMixin):
    """
    Root IResource implementation for unauthenticated users.

    This resource allows users to login, reset their passwords, or access
    content provided by any site root plugins.
    """
    powerupInterfaces = (IResource, IMantissaSite, IWebViewer)
    implements(*powerupInterfaces + (IPowerupIndirector,))

    schemaVersion = 2

    loginSystem = dependsOn(userbase.LoginSystem)


    def rootChild_resetPassword(self, req, webViewer):
        """
        Return a page which will allow the user to re-set their password.
        """
        from xmantissa.signup import PasswordResetResource
        return PasswordResetResource(self.store)


    def rootChild_login(self, req, webViewer):
        """
        Return a login page.
        """
        return LoginPage(self.store)


    def rootChild_users(self, req, webViewer):
        """
        Return a child resource to provide access to items shared by users.

        @return: a resource whose children will be private pages of individual
        users.

        @rtype L{xmantissa.websharing.UserIndexPage}
        """
        return UserIndexPage(self.loginSystem, webViewer)


    def _getUsername(self):
        """
        Inform L{VirtualHostWrapper} that it's being accessed anonymously.
        """
        return None


    # IPowerupIndirector
    def indirect(self, interface):
        """
        Indirect the implementation of L{IWebViewer} to L{_AnonymousWebViewer}.
        """
        if interface == IWebViewer:
            return _AnonymousWebViewer(self.store)
        return super(AnonymousSite, self).indirect(interface)



AnonymousSite1 = item.declareLegacyItem(
    'xmantissa_publicweb_anonymoussite', 1,
    dict(
        loginSystem=attributes.reference(),
    ))


def _installV2Powerups(anonymousSite):
    """
    Install the given L{AnonymousSite} for the powerup interfaces it was given
    in version 2.
    """
    anonymousSite.store.powerUp(anonymousSite, IWebViewer)
    anonymousSite.store.powerUp(anonymousSite, IMantissaSite)
upgrade.registerAttributeCopyingUpgrader(AnonymousSite, 1, 2, _installV2Powerups)




from OpenSSL.crypto import FILETYPE_PEM

from twisted.internet.ssl import PrivateCertificate, KeyPair

from axiom.item import Item
from axiom.attributes import text
from axiom.dependency import installOn
from axiom.test.historic.stubloader import saveStub

from xmantissa.port import TCPPort, SSLPort
from xmantissa.website import WebSite

# Unfortunately, the test module for this store binds ports.  So pick some
# improbably port numbers and hope they aren't bound.  If they are, the test
# will fail.  Hooray! -exarkun
TCP_PORT = 29415
SSL_PORT = 19224

def createDatabase(siteStore):
    """
    Populate the given Store with a TCPPort and SSLPort.
    """
    factory = WebSite(store=siteStore)
    installOn(factory, siteStore)
    installOn(
        TCPPort(store=siteStore, portNumber=TCP_PORT, factory=factory),
        siteStore)
    certificatePath = siteStore.newFilePath('certificate')

    key = KeyPair.generate()
    cert = key.selfSignedCert(1)
    certificatePath.setContent(
        cert.dump(FILETYPE_PEM) +
        key.dump(FILETYPE_PEM))

    installOn(
        SSLPort(store=siteStore, portNumber=SSL_PORT,
                certificatePath=certificatePath,
                factory=factory),
        siteStore)



if __name__ == '__main__':
    saveStub(createDatabase, 12731)



# -*- test-case-name: xmantissa.test.test_websharing -*-
# Copyright 2008 Divmod, Inc. See LICENSE file for details
"""
This module provides web-based access to objects shared with the
xmantissa.sharing module.

Users' publicly shared objects are exposed at the url::

    http://your-server/users/<user>@<hostname>/<share-id>

Applications' publicly shared objects are exposed at the url::

    http://your-server/<app-name>/<share-id>

where "app-name" is the name of the offering the application was installed
from.

To share an item publicly, share it with the "everyone" role. To place it at
the root URL of the share location, call L{addDefaultShareID} with its share
ID.

Example::
  sharing.getEveryoneRole(yourItem.store).shareItem(yourItem, shareID=u'bob')

If this is in an app store named 'foo', this object is now shared on the URL
http://your-server/foo/bob. To share it at the root, as
http://your-server/foo/, make it the default::
  websharing.addDefaultShareID(yourItem.store, u'bob', 1)

"""

from zope.interface import implements

from axiom import userbase, attributes
from axiom.item import Item
from axiom.attributes import text, integer

from nevow import inevow, url, rend

from xmantissa.offering import isAppStore

from xmantissa import sharing

class _DefaultShareID(Item):
    """
    Item which holds a default share ID for a user's store.  Default share IDs
    are associated with a priority, and the highest-priority ID identifies the
    share which will be selected if the user browsing a substore doesn't
    provide their own share ID.
    """
    shareID = text(doc="""
    A default share ID for the store this item lives in.
    """, allowNone=False)

    priority = integer(doc="""
    The priority of this default.  Higher means more important.
    """)



def addDefaultShareID(store, shareID, priority):
    """
    Add a default share ID to C{store}, pointing to C{shareID} with a
    priority C{priority}.  The highest-priority share ID identifies the share
    that will be retrieved when a user does not explicitly provide a share ID
    in their URL (e.g. /host/users/username/).

    @param shareID: A share ID.
    @type shareID: C{unicode}

    @param priority: The priority of this default.  Higher means more
    important.
    @type priority: C{int}
    """
    _DefaultShareID(store=store, shareID=shareID, priority=priority)



def getDefaultShareID(store):
    """
    Get the highest-priority default share ID for C{store}.

    @return: the default share ID, or u'' if one has not been set.
    @rtype: C{unicode}
    """
    defaultShareID = store.findFirst(
        _DefaultShareID, sort=_DefaultShareID.priority.desc)
    if defaultShareID is None:
        return u''
    return defaultShareID.shareID



class _ShareURL(url.URL):
    """
    An L{url.URL} subclass which inserts share ID as a path segment in the URL
    just before the first call to L{child} modifies it.
    """
    def __init__(self, shareID, *a, **k):
        """
        @param shareID: The ID of the share we are generating a URL for.
        @type shareID: C{unicode}.
        """
        self._shareID = shareID
        url.URL.__init__(self, *a, **k)


    def child(self, path):
        """
        Override the base implementation to inject the share ID our
        constructor was passed.
        """
        if self._shareID is not None:
            self = url.URL.child(self, self._shareID)
            self._shareID = None
        return url.URL.child(self, path)


    def cloneURL(self, scheme, netloc, pathsegs, querysegs, fragment):
        """
        Override the base implementation to pass along the share ID our
        constructor was passed.
        """
        return self.__class__(
            self._shareID, scheme, netloc, pathsegs, querysegs, fragment)


    # there is no point providing an implementation of any these methods which
    # accepts a shareID argument; they don't really mean anything in this
    # context.

    def fromString(cls, string):
        """
        Override the base implementation to throw L{NotImplementedError}.

        @raises L{NotImplementedError}: always.
        """
        raise NotImplementedError(
            'fromString is not implemented on %r' % (cls.__name__,))
    fromString = classmethod(fromString)


    def fromRequest(cls, req):
        """
        Override the base implementation to throw L{NotImplementedError}.

        @raises L{NotImplementedError}: always.
        """
        raise NotImplementedError(
            'fromRequest is not implemented on %r' % (cls.__name__,))
    fromRequest = classmethod(fromRequest)


    def fromContext(cls, ctx):
        """
        Override the base implementation to throw L{NotImplementedError}.

        @raises L{NotImplementedError}: always.
        """
        raise NotImplementedError(
            'fromContext is not implemented on %r' % (cls.__name__,))
    fromContext = classmethod(fromContext)



def linkTo(sharedProxyOrItem):
    """
    Generate the path part of a URL to link to a share item or its proxy.

    @param sharedProxy: a L{sharing.SharedProxy} or L{sharing.Share}

    @return: a URL object, which when converted to a string will look
        something like '/users/user@host/shareID'.

    @rtype: L{nevow.url.URL}

    @raise: L{RuntimeError} if the store that the C{sharedProxyOrItem} is
        stored in is not accessible via the web, for example due to the fact
        that the store has no L{LoginMethod} objects to indicate who it is
        owned by.
    """
    if isinstance(sharedProxyOrItem, sharing.SharedProxy):
        userStore = sharing.itemFromProxy(sharedProxyOrItem).store
    else:
        userStore = sharedProxyOrItem.store
    appStore = isAppStore(userStore)
    if appStore:
        # This code-path should be fixed by #2703; PublicWeb is deprecated.
        from xmantissa.publicweb import PublicWeb
        substore = userStore.parent.getItemByID(userStore.idInParent)
        pw = userStore.parent.findUnique(PublicWeb, PublicWeb.application == substore)
        path = [pw.prefixURL.encode('ascii')]
    else:
        for lm in userbase.getLoginMethods(userStore):
            if lm.internal:
                path = ['users', lm.localpart.encode('ascii')]
                break
        else:
            raise RuntimeError(
                "Shared item is in a user store with no"
                " internal username -- can't generate a link.")
    if (sharedProxyOrItem.shareID == getDefaultShareID(userStore)):
        shareID = sharedProxyOrItem.shareID
        path.append('')
    else:
        shareID = None
        path.append(sharedProxyOrItem.shareID)
    return _ShareURL(shareID, scheme='', netloc='', pathsegs=path)



def _storeFromUsername(store, username):
    """
    Find the user store of the user with username C{store}

    @param store: site-store
    @type store: L{axiom.store.Store}

    @param username: the name a user signed up with
    @type username: C{unicode}

    @rtype: L{axiom.store.Store} or C{None}
    """
    lm = store.findUnique(
            userbase.LoginMethod,
            attributes.AND(
                userbase.LoginMethod.localpart == username,
                userbase.LoginMethod.internal == True),
            default=None)
    if lm is not None:
        return lm.account.avatars.open()



class UserIndexPage(object):
    """
    This is the resource accessible at "/users"

    See L{xmantissa.publicweb.AnonymousSite.child_users} for the integration
    point with the rest of the system.
    """
    implements(inevow.IResource)

    def __init__(self, loginSystem, webViewer):
        """
        Create a UserIndexPage which draws users from a given
        L{userbase.LoginSystem}.

        @param loginSystem: the login system to look up users in.
        @type loginSystem: L{userbase.LoginSystem}
        """
        self.loginSystem = loginSystem
        self.webViewer = webViewer


    def locateChild(self, ctx, segments):
        """
        Retrieve a L{SharingIndex} for a particular user, or rend.NotFound.
        """
        store = _storeFromUsername(
            self.loginSystem.store, segments[0].decode('utf-8'))
        if store is None:
            return rend.NotFound
        return (SharingIndex(store, self.webViewer), segments[1:])


    def renderHTTP(self, ctx):
        """
        Return a sarcastic string to the user when they try to list the index of
        users by hitting '/users' by itself.

        (This should probably do something more helpful.  There might be a very
        large number of users so returning a simple listing is infeasible, but
        one might at least present a search page or something.)
        """
        return 'Keep trying.  You are almost there.'



class SharingIndex(object):
    """
    A SharingIndex is an http resource which provides a view onto a user's
    store, for another user.
    """
    implements(inevow.IResource)

    def __init__(self, userStore, webViewer):
        """
        Create a SharingIndex.

        @param userStore: an L{axiom.store.Store} to be viewed.

        @param webViewer: an L{IWebViewer} which represents the
        viewer.
        """
        self.userStore = userStore
        self.webViewer = webViewer


    def renderHTTP(self, ctx):
        """
        When this resource itself is rendered, redirect to the default shared
        item.
        """
        return url.URL.fromContext(ctx).child('')


    def locateChild(self, ctx, segments):
        """
        Look up a shared item for the role viewing this SharingIndex and return a
        L{PublicAthenaLivePage} containing that shared item's fragment to the
        user.

        These semantics are UNSTABLE.  This method is adequate for simple uses,
        but it should be expanded in the future to be more consistent with
        other resource lookups.  In particular, it should allow share
        implementors to adapt their shares to L{IResource} directly rather than
        L{INavigableFragment}, to allow for simpler child dispatch.

        @param segments: a list of strings, the first of which should be the
        shareID of the desired item.

        @param ctx: unused.

        @return: a L{PublicAthenaLivePage} wrapping a customized fragment.
        """
        shareID = segments[0].decode('utf-8')

        role = self.webViewer.roleIn(self.userStore)

        # if there is an empty segment
        if shareID == u'':
            # then we want to return the default share.  if we find one, then
            # let's use that
            defaultShareID = getDefaultShareID(self.userStore)
            try:
                sharedItem = role.getShare(defaultShareID)
            except sharing.NoSuchShare:
                return rend.NotFound
        # otherwise the user is trying to access some other share
        else:
            # let's see if it's a real share
            try:
                sharedItem = role.getShare(shareID)
            # oops it's not
            except sharing.NoSuchShare:
                return rend.NotFound

        return (self.webViewer.wrapModel(sharedItem),
                segments[1:])



# -*- test-case-name: xmantissa.test.historic.test_userinfo1to2 -*-

"""
Create the stub database for the test for the UserInfo schema version 1 to 2
upgrader.
"""

from axiom.test.historic.stubloader import saveStub

from xmantissa.signup import UserInfo

FIRST = u'Alice'
LAST = u'Smith'

def createDatabase(store):
    """
    Create a version 1 L{UserInfo} item in the given store.
    """
    UserInfo(store=store, firstName=FIRST, lastName=LAST)


if __name__ == '__main__':
    saveStub(createDatabase, 13447)



from axiom.test.historic import stubloader
from xmantissa.people import AddPerson, Organizer
from xmantissa.webapp import PrivateApplication

class AddPersonTest(stubloader.StubbedTest):
    def testUpgrade(self):
        ap = self.store.findUnique(AddPerson)
        self.failUnless(isinstance(ap.organizer, Organizer))



from axiom.test.historic import stubloader
from xmantissa.people import EmailAddress, Person

class EmailAddressTestCase(stubloader.StubbedTest):
    def testUpgrade(self):
        ea = self.store.findUnique(EmailAddress)
        person = self.store.findUnique(Person)
        self.assertIdentical(ea.person, person)
        self.assertEquals(ea.address, 'bob@divmod.com')



from twisted.python.filepath import FilePath

from axiom.test.historic.stubloader import saveStub
from xmantissa.fulltext import PyLuceneIndexer

def createDatabase(s):
    PyLuceneIndexer(store=s,
                    installedOn=s,
                    indexCount=23,
                    indexDirectory=u'foo.index').installOn(s)

if __name__ == '__main__':
    saveStub(createDatabase, 9044)



import email

from twisted.trial.unittest import TestCase

from nevow.url import URL
from nevow.flat import flatten
from nevow.inevow import IResource
from nevow.testutil import FakeRequest, renderPage

from axiom.store import Store
from axiom import userbase
from axiom.dependency import installOn
from axiom.plugins.mantissacmd import Mantissa

from xmantissa import ixmantissa
from xmantissa.web import SiteConfiguration
from xmantissa.webapp import PrivateApplication
from xmantissa.signup import PasswordResetResource, _PasswordResetAttempt


class PasswordResetTestCase(TestCase):
    def setUp(self):
        """
        Set up a fake objects and methods for the password reset tests.
        """
        siteStore = Store(filesdir=self.mktemp())
        Mantissa().installSite(siteStore, u"divmod.com", u"", False)
        self.loginSystem = siteStore.findUnique(userbase.LoginSystem)
        self.site = siteStore.findUnique(SiteConfiguration)

        la = self.loginSystem.addAccount(
            u'joe', u'divmod.com', u'secret', internal=True)

        la.addLoginMethod(
            u'joe', u'external.com',
            protocol=u'zombie dance',
            verified=False,
            internal=False)

        # create an account with no external mail address
        account = self.loginSystem.addAccount(
            u'jill', u'divmod.com', u'secret', internal=True)

        account.addLoginMethod(
            u'jill', u'divmod.com',
            protocol=u'zombie dance',
            verified=True,
            internal=True)

        self.siteStore = siteStore

        # Set up the user store to have all the elements necessary to redirect
        # in the case where the user is already logged in.
        substore = la.avatars.open()
        installOn(PrivateApplication(store=substore), substore)

        self.userStore = substore
        self.loginAccount = la
        self.nonExternalAccount = account
        self.reset = PasswordResetResource(self.siteStore)


    def test_reset(self):
        """
        Test a password reset, as it might happen for a user
        """
        self.reset.resetPassword(
            self.reset.newAttemptForUser(u'joe@divmod.com'),
            u'more secret')

        self.assertEqual(self.loginAccount.password, u'more secret')
        self.assertEqual(
            self.siteStore.query(_PasswordResetAttempt).count(), 0)


    def test_attemptByKey(self):
        """
        Test that L{xmantissa.signup.PasswordResetResource.attemptByKey}
        knows the difference between valid and invalid keys
        """
        self.failUnless(self.reset.attemptByKey(
                self.reset.newAttemptForUser(u'joe@divmod.com').key))
        self.failIf(self.reset.attemptByKey(u'not really a key'))


    def test_accountByAddress(self):
        """
        Test that L{xmantissa.signup.PasswordResetResource.accountByAddress}
        behaves similarly to L{axiom.userbase.LoginSystem.accountByAddress}
        """
        self.assertEqual(
            self.reset.accountByAddress(u'joe@divmod.com'),
            self.loginSystem.accountByAddress(u'joe', u'divmod.com'))


    def test_handleRequest(self):
        """
        Check that handling a password reset request for a user sends email
        appropriately.
        """
        def myFunc(url, attempt, email):
            myFunc.emailsSent += 1
            myFunc.url = url
            myFunc.attempt = attempt
            myFunc.email = email
        url = 'http://oh.no/reset.html'
        myFunc.emailsSent = 0
        self.reset.sendEmail=myFunc

        # positive case. User exists. Email should be sent
        self.reset.handleRequestForUser(u'joe@divmod.com', url)
        self.assertEquals(myFunc.emailsSent, 1)
        self.assertEquals(myFunc.url, 'http://oh.no/reset.html')
        self.assertEquals(myFunc.attempt.username, u'joe@divmod.com')
        self.assertEquals(myFunc.email, 'joe@external.com')

        # Negative case. User does not exist. Email should not be sent
        self.reset.handleRequestForUser(u'no_joe@divmod.com', url)
        self.assertEquals(myFunc.emailsSent, 1)

        # Negative case. User exists, but has no external mail. Email should not
        # be sent.
        self.reset.handleRequestForUser(u'jill@divmod.com', url)
        self.assertEquals(myFunc.emailsSent, 1)


    def test_sendEmail(self):
        """
        L{PasswordResetResource.sendEmail} should format a meaningful password
        reset email.
        """
        resetAddress = 'reset@example.org'
        resetURI = URL.fromString('http://example.org/resetPassword')
        userAddress = 'joe@divmod.com'

        resetAttempt = self.reset.newAttemptForUser(userAddress.decode('ascii'))
        _sentEmail = []
        self.reset.sendEmail(resetURI, resetAttempt, userAddress,
                             _sendEmail=lambda *args: _sentEmail.append(args))

        self.assertEquals(len(_sentEmail), 1)
        [(sentFrom, sentTo, sentText)] = _sentEmail
        self.assertEquals(sentFrom, resetAddress)
        self.assertEquals(sentTo, userAddress)

        msg = email.message_from_string(sentText)
        [headerFrom] = msg.get_all('from')
        [headerTo] = msg.get_all('to')
        [headerDate] = msg.get_all('date')
        # Python < 2.5 compatibility
        try:
            from email import utils
        except ImportError:
            from email import Utils as utils
        self.assertEquals(utils.parseaddr(headerFrom)[1], resetAddress)
        self.assertEquals(utils.parseaddr(headerTo)[1], userAddress)
        self.assertTrue(utils.parsedate_tz(headerDate) is not None,
                        '%r is not a RFC 2822 date' % headerDate)

        self.assertTrue(not msg.is_multipart())
        self.assertIn(flatten(resetURI.child(resetAttempt.key)),
                      msg.get_payload())


    def test_redirectToSettingsWhenLoggedIn(self):
        """
        When a user is already logged in, navigating to /resetPassword should
        redirect to the settings page, since the user can change their password
        from there.
        """
        self.assertNotIdentical(self.userStore.parent, None) # sanity check
        prefPage = ixmantissa.IPreferenceAggregator(self.userStore)
        urlPath = ixmantissa.IWebTranslator(self.userStore).linkTo(prefPage.storeID)
        request = FakeRequest(headers={"host": "example.com"})
        app = IResource(self.userStore)
        rsc = IResource(app.locateChild(request, ("resetPassword",))[0])
        d = renderPage(rsc, reqFactory=lambda : request)
        def rendered(result):
            self.assertEquals(
                'http://example.com' + urlPath,
                request.redirected_to)
        d.addCallback(rendered)
        return d


    def test_getExternalEmail(self):
        """
        Test that we can accurately retrieve an external email address from an
        attempt.
        """
        email = self.reset.getExternalEmail(self.loginAccount)
        self.assertEquals(email, 'joe@external.com')


    def test_noExternalEmail(self):
        """
        Test that C{getExternalEmail} returns C{None} if there is no external
        email address for that account.
        """
        email = self.reset.getExternalEmail(self.nonExternalAccount)
        self.assertEquals(email, None)


    def test_nothingSpecified(self):
        """
        Submitting an empty form should redirect the user back to the form.
        """
        self.reset.handleRequestForUser = lambda *args: self.fail(args)

        _request = FakeRequest(
            headers={'host': 'example.org'},
            uri='/resetPassword',
            currentSegments=['resetPassword'],
            args={'username': [''], 'email': ['']})
        _request.method = 'POST'

        d = renderPage(self.reset, reqFactory=lambda: _request)
        def rendered(_):
            self.assertEquals(_request.redirected_to,
                              'http://example.org/resetPassword')
        d.addCallback(rendered)
        return d


    def test_onlyUsernameSpecified(self):
        """
        Test that if the user only supplies the local part of their username
        then the password resetter will still find the correct user.
        """
        hostname = self.site.hostname.encode('ascii')

        def handleRequest(username, url):
            handleRequest.attempt = self.reset.newAttemptForUser(username)
            handleRequest.username = username

        class Request(FakeRequest):
            method = 'POST'

            def __init__(self, *a, **kw):
                FakeRequest.__init__(self, *a, **kw)
                self.args = {'username': ['joe'],
                             'email': ['']}
                self.received_headers['host'] = hostname

        self.reset.handleRequestForUser = handleRequest
        d = renderPage(self.reset, reqFactory=Request)
        d.addCallback(lambda _: self.assertEquals(handleRequest.username,
                                                  'joe@' + hostname))
        return d


    def test_emailAddressSpecified(self):
        """
        If an email address and no username is specified, then the password
        resetter should still find the correct user.
        """
        requests = []
        def handleRequest(username, url):
            requests.append((username, url))

        class Request(FakeRequest):
            method = 'POST'
            def __init__(self, *a, **k):
                FakeRequest.__init__(self, *a, **k)
                self.args = {'username': [''],
                             'email': ['joe@external.com']}

        self.reset.handleRequestForUser = handleRequest
        d = renderPage(self.reset, reqFactory=Request)
        def completedRequest():
            self.assertEqual(len(requests), 1)
            self.assertEqual(requests[0][0], 'joe@divmod.com')
        d.addCallback(lambda ign: completedRequest())
        return d


    def specifyBogusEmail(self, bogusEmail):
        """
        If an email address (which is not present in the system) and no
        username is specified, then the password reset should still ask the
        user to check their email.  No distinction is provided to discourage
        "oafish attempts at hacking", as duncan poetically put it.
        """
        requests = []
        def handleRequest(username, url):
            requests.append((username, url))

        class Request(FakeRequest):
            method = 'POST'
            def __init__(self, *a, **k):
                FakeRequest.__init__(self, *a, **k)
                self.args = {'username': [''],
                             'email': [bogusEmail]}

        self.reset.handleRequestForUser = handleRequest
        d = renderPage(self.reset, reqFactory=Request)
        def completedRequest():
            self.assertEqual(len(requests), 0)
        d.addCallback(lambda ign: completedRequest())
        return d


    def test_notPresentEmailAddress(self):
        """
        If an email address is not present in the system, no notification
        should be sent, but the user should receive the same feedback as if it
        worked, to discourage cracking attempts.
        """
        return self.specifyBogusEmail('not-in-the-system@example.com')


    def test_malformedEmailAddress(self):
        """
        If a malformed email address is provided, no notification should be
        sent, but the user should receive the same feedback as if it worked, to
        discourage cracking attempts.
        """
        return self.specifyBogusEmail('hello, world!')



# -*- test-case-name: xmantissa.test -*-

"""
Exception definitions for Mantissa.
"""

from nevow.errors import MissingDocumentFactory


class ArgumentError(Exception):
    """
    Base class for all exceptions raised by the address parser due to malformed
    input.
    """



class AddressTooLong(ArgumentError):
    """
    Exception raised when an address which exceeds the maximum allowed length
    is given to the parser.
    """



class InvalidAddress(ArgumentError):
    """
    Exception raised when an address is syntactically invalid.
    """



class InvalidTrailingBytes(ArgumentError):
    """
    Exception raised when there are extra bytes at the end of an address.
    """



class Unsortable(Exception):
    """
    This exception is raised when a client invalidly attempts to sort a table
    view by a column that is not available for sorting.
    """



class MessageTransportError(Exception):
    """
    A message transport failed in some unrecoverable way; the transmission
    needs to be retried.
    """



class BadSender(Exception):
    """
    A substore attempted to send a message for a username / domain pair for
    which it was not authorized.

    @ivar attemptedSender: A unicode string, formatted as user@host, that
    indicates the user ID that the system attempted to send a message from.

    @ivar allowedSenders: A list of unicode strings, formatted like
    C{attemptedSender}, indicating the senders that the system is allowed to
    send messages as.
    """
    def __init__(self, attemptedSender, allowedSenders):
        """
        Create a L{BadSender} exception with the list of attempted senders and
        the list of allowed senders.
        """
        self.attemptedSender = attemptedSender
        self.allowedSenders = allowedSenders
        Exception.__init__(self, allowedSenders[0].encode('utf-8') +
                           " attempted to send message as " +
                           self.attemptedSender.encode('utf-8'))



class UnknownMessageType(Exception):
    """
    A message of an unknown type was received.
    """



class MalformedMessage(Exception):
    """
    A message of the AMP message type was received, but its body could not be
    parsed as a single AMP box.
    """



class RevertAndRespond(Exception):
    """
    This special exception type allows an
    L{xmantissa.ixmantissa.IMessageReceiver.messageReceived} implementation to
    generate an answer, but revert the transaction where the application is
    processing the application logic.

    @ivar value: the L{Value} of the answer to be issued.
    """

    def __init__(self, value):
        """
        Create a L{RevertAndRespond} with an answer that will be provided to
        the message sender.
        """
        self.value = value



class CouldNotLoadFromThemes(MissingDocumentFactory):
    """
    An L{INavigableFragment} didn't have a C{docFactory}, and also didn't have
    a C{fragmentName} that could be used to load one from a theme.  This
    exception should help developers with debugging when they forget to give a
    docFactory or a fragmentName to an element.
    """

    def __init__(self, element, themes):
        """
        @param element: The L{INavigableFragment} that was being rendered.

        @param themes: A list of L{ITemplateNameResolver}s.
        """
        MissingDocumentFactory.__init__(self, element)
        self.themes = themes


    def __repr__(self):
        """
        A string representation including helpful information about the themes
        that were searched and the element which could not be themed.
        """
        return ("%s: %r (fragment name %r) has no docFactory. Searched these themes: %r"
                % (self.__class__.__name__, self.element,
                   self.element.fragmentName, self.themes))



from zope.interface import implements

from twisted.trial.unittest import TestCase
from twisted.internet import defer

from epsilon.structlike import record

from axiom.store import Store
from axiom.item import Item
from axiom.attributes import integer
from axiom.substore import SubStore
from axiom.dependency import installOn
from axiom.plugins.axiom_plugins import Create
from axiom.plugins.mantissacmd import Mantissa

from nevow.athena import LiveElement
from nevow import rend
from nevow.rend import WovenContext
from nevow.testutil import FakeRequest
from nevow.inevow import IRequest, IResource

from xmantissa.ixmantissa import (
    ITemplateNameResolver, ISiteURLGenerator, IWebViewer, INavigableElement)

from xmantissa.offering import InstalledOffering
from xmantissa.webtheme import theThemeCache
from xmantissa.webnav import Tab
from xmantissa.sharing import (getSelfRole, getAuthenticatedRole,
                               getPrimaryRole)

from xmantissa.webapp import (
    PrivateApplication, _AuthenticatedWebViewer, _PrivateRootPage,
    GenericNavigationAthenaPage)

from xmantissa.test.test_publicweb import AuthenticatedNavigationTestMixin

from xmantissa.test.fakes import (
    FakeTheme, FakeCustomizableElementModel, FakeLoader,
    ElementViewForFakeModelWithTheme, FakeElementModelWithTheme)

from xmantissa.test.test_webshell import WebViewerTestMixin


class AuthenticatedWebViewerTests(WebViewerTestMixin, TestCase):
    """
    Tests for L{_AuthenticatedWebViewer}.
    """

    def setupPageFactory(self):
        """
        Create the page factory used by the tests.
        """
        self.privapp = self.adminStore.findUnique(PrivateApplication)
        self.pageFactory = _AuthenticatedWebViewer(self.privapp)


    def assertPage(self, page):
        """
        Fail if the given object is not a nevow L{rend.Page} (or if it is an
        L{LivePage}). Also fail if the username is wrong.
        """
        WebViewerTestMixin.assertPage(self, page)
        self.assertEqual(page.username, u'admin@localhost')


    def assertLivePage(self, page):
        """
        Fail if the given object is not a L{MantissaLivePage}, or if the
        username is wrong.
        """
        WebViewerTestMixin.assertLivePage(self, page)
        self.assertEqual(page.username, u'admin@localhost')



    def test_docFactoryFromFragmentNameWithPreference(self):
        """
        When an L{INavigableFragment} provider provides a C{fragmentName}
        attribute, the theme to load it should be discovered according to the
        user's preference.
        """
        preferredDocFactory = FakeLoader('good')
        otherDocFactory = FakeLoader('bad')
        fn = ElementViewForFakeModelWithTheme.fragmentName
        self.stubThemeList(
            [FakeTheme(u'alpha', {fn: otherDocFactory}),
             FakeTheme(u'beta', {fn: preferredDocFactory})])
        self.privapp.preferredTheme = u'beta'
        elementable = FakeElementModelWithTheme()
        result = self.pageFactory.wrapModel(elementable)
        self.assertEqual(result.fragment.docFactory, preferredDocFactory)


    def test_customizableCustomizeFor(self):
        """
        L{INavigableFragment} providers who have a 'customizeFor' method will
        have it called with a username when they are wrapped for authenticated
        rendering.
        """
        elementable = FakeCustomizableElementModel()
        result = self.pageFactory.wrapModel(elementable)
        self.assertEqual(elementable.username, u'admin@localhost')


    def test_roleInMyStore(self):
        """
        L{_AuthenticatedWebViewer} should always return the 'self' role for
        users looking at their own stores.
        """
        role = getSelfRole(self.adminStore)
        self.assertIdentical(self.pageFactory.roleIn(self.adminStore),
                             role)


    def test_roleInSomebodyElsesStoreWhoDoesntKnowMe(self):
        """
        L{_AuthenticatedWebViewer} should return the authenticated role for
        users with no specific role to map.
        """
        someStore = self.loginSystem.addAccount(
            u'someguy', u'localhost', u'asdf').avatars.open()
        role = getAuthenticatedRole(someStore)
        self.assertIdentical(self.pageFactory.roleIn(someStore),
                             role)


    def test_roleInSomebodyElsesStoreDoesKnowMe(self):
        """
        L{_AuthenticatedWebViewer} should return the authenticated role for
        users with no specific role to map.
        """
        someStore = self.loginSystem.addAccount(
            u'someguy', u'localhost', u'asdf').avatars.open()
        role = getPrimaryRole(someStore, u'admin@localhost', True)
        self.assertIdentical(self.pageFactory.roleIn(someStore),
                             role)



class FakeResourceItem(Item):
    unused = integer()
    implements(IResource)

class FakeModelItem(Item):
    unused = integer()


class WebIDLocationTest(TestCase):

    def setUp(self):
        store = Store()
        ss = SubStore.createNew(store, ['test']).open()
        self.pa = PrivateApplication(store=ss)
        installOn(self.pa, ss)
        self.webViewer = IWebViewer(ss)

    def test_powersUpTemplateNameResolver(self):
        """
        L{PrivateApplication} implements L{ITemplateNameResolver} and should
        power up the store it is installed on for that interface.
        """
        self.assertIn(
            self.pa,
            self.pa.store.powerupsFor(ITemplateNameResolver))


    def test_suchWebID(self):
        """
        Verify that retrieving a webID gives the correct resource.
        """
        i = FakeResourceItem(store=self.pa.store)
        wid = self.pa.toWebID(i)
        ctx = FakeRequest()
        res = self.pa.createResourceWith(self.webViewer)
        self.assertEqual(res.locateChild(ctx, [wid]),
                         (i, []))


    def test_noSuchWebID(self):
        """
        Verify that non-existent private URLs generate 'not found' responses.
        """
        ctx = FakeRequest()
        for segments in [
            # something that looks like a valid webID
            ['0000000000000000'],
            # something that doesn't
            ["nothing-here"],
            # more than one segment
            ["two", "segments"]]:
            res = self.pa.createResourceWith(self.webViewer)
            self.assertEqual(res.locateChild(ctx, segments),
                             rend.NotFound)


    def test_webIDForFragment(self):
        """
        Retrieving a webID that specifies a fragment gives the correct
        resource.
        """
        class FakeView(record("model")):
            "A fake view that wraps a FakeModelItem."

        class FakeWebViewer(object):
            def wrapModel(self, model):
                return FakeView(model)

        i = FakeModelItem(store=self.pa.store)
        wid = self.pa.toWebID(i)
        ctx = FakeRequest()
        res = self.pa.createResourceWith(FakeWebViewer())
        child, segs = res.locateChild(ctx, [wid])
        self.assertIsInstance(child, FakeView)
        self.assertIdentical(child.model, i)



class TestElement(LiveElement):
    def head(self):
        pass

    def locateChild(self, ctx, segs):
        if segs[0] == 'child-of-fragment':
            return ('I AM A CHILD OF THE FRAGMENT', segs[1:])
        return rend.NotFound


class TestClientFactory(object):
    """
    Dummy L{LivePageFactory}.

    @ivar magicSegment: The segment for which to return L{returnValue} from
    L{getClient}.
    @type magicSegment: C{str}

    @ivar returnValue: The value to return from L{getClient} when it is passed
    L{magicSegment}.
    @type returnValue: C{str}.
    """
    def __init__(self, magicSegment, returnValue):
        self.magicSegment = magicSegment
        self.returnValue = returnValue


    def getClient(self, seg):
        if seg == self.magicSegment:
            return self.returnValue


class GenericNavigationAthenaPageTests(TestCase,
                                       AuthenticatedNavigationTestMixin):
    """
    Tests for L{GenericNavigationAthenaPage}.
    """
    def setUp(self):
        """
        Set up a site store, user store, and page instance to test with.
        """
        self.siteStore = Store(filesdir=self.mktemp())
        def siteStoreTxn():
            Mantissa().installSite(self.siteStore, u"localhost", u"", False)

            self.userStore = SubStore.createNew(
                self.siteStore, ['child', 'lookup']).open()
        self.siteStore.transact(siteStoreTxn)

        def userStoreTxn():
            self.privateApp = PrivateApplication(store=self.userStore)
            installOn(self.privateApp, self.userStore)

            self.navpage = self.createPage(None)
        self.userStore.transact(userStoreTxn)


    def createPage(self, username):
        """
        Create a L{GenericNavigationAthenaPage} for the given user.
        """
        return GenericNavigationAthenaPage(
            self.privateApp,
            TestElement(),
            self.privateApp.getPageComponents(),
            username)


    def rootURL(self, request):
        """
        Return the root URL as reported by C{self.website}.
        """
        return ISiteURLGenerator(self.siteStore).rootURL(request)


    def test_childLookup(self):
        """
        L{GenericNavigationAthenaPage} should delegate to its fragment and its
        L{LivePageFactory} when it cannot find a child itself.
        """
        self.navpage.factory = tcf = TestClientFactory(
            'client-of-livepage', 'I AM A CLIENT OF THE LIVEPAGE')

        self.assertEqual(self.navpage.locateChild(None,
                                                 ('child-of-fragment',)),
                         ('I AM A CHILD OF THE FRAGMENT', ()))
        self.assertEqual(self.navpage.locateChild(None,
                                             (tcf.magicSegment,)),
                        (tcf.returnValue, ()))


    def test_jsModuleLocation(self):
        """
        L{GenericNavigationAthenaPage.beforeRender} should should call
        L{xmantissa.website.MantissaLivePage.beforeRender}, which shares its
        Athena JavaScript module location with all other pages that use
        L{xmantissa.cachejs}, and provide links to /__jsmodule__/.
        """
        ctx = WovenContext()
        req = FakeRequest()
        ctx.remember(req, IRequest)
        self.navpage.beforeRender(ctx)
        urlObj = self.navpage.getJSModuleURL('Mantissa')
        self.assertEqual(urlObj.pathList()[0], '__jsmodule__')


    def test_beforeRenderDelegation(self):
        """
        L{GenericNavigationAthenaPage.beforeRender} should call
        C{beforeRender} on the wrapped fragment, if it's defined, and return
        its result.
        """
        contexts = []
        result = defer.succeed(None)
        def beforeRender(ctx):
            contexts.append(ctx)
            return result
        self.navpage.fragment.beforeRender = beforeRender
        ctx = WovenContext()
        ctx.remember(FakeRequest(), IRequest)
        self.assertIdentical(
            self.navpage.beforeRender(ctx), result)
        self.assertEqual(contexts, [ctx])



class PrivateApplicationTestCase(TestCase):
    """
    Tests for L{PrivateApplication}.
    """
    def setUp(self):
        self.siteStore = Store(filesdir=self.mktemp())
        Mantissa().installSite(self.siteStore, u"example.com", u"", False)

        self.userAccount = Create().addAccount(
            self.siteStore, u'testuser', u'example.com', u'password')
        self.userStore = self.userAccount.avatars.open()

        self.privapp = PrivateApplication(store=self.userStore)
        installOn(self.privapp, self.userStore)
        self.webViewer = IWebViewer(self.userStore)


    def test_createResourceUsername(self):
        """
        L{PrivateApplication.createResourceWith} should figure out the
        right username and pass it to L{_PrivateRootPage}.
        """
        rootPage = self.privapp.createResourceWith(self.webViewer)
        self.assertEqual(rootPage.username, u'testuser@example.com')


    def test_getDocFactory(self):
        """
        L{PrivateApplication.getDocFactory} finds a document factory for
        the specified template name from among the installed themes.
        """
        # Get something from the Mantissa theme
        self.assertNotIdentical(self.privapp.getDocFactory('shell'), None)

        # Get rid of the Mantissa offering and make sure the template is no
        # longer found.
        self.siteStore.query(InstalledOffering).deleteFromStore()

        # And flush the cache. :/ -exarkun
        theThemeCache.emptyCache()

        self.assertIdentical(self.privapp.getDocFactory('shell'), None)


    def test_powersUpWebViewer(self):
        """
        L{PrivateApplication} should provide an indirected L{IWebViewer}
        powerup, and its indirected powerup should be the default provider of
        that interface.
        """
        webViewer = IWebViewer(self.privapp.store)
        self.assertIsInstance(webViewer, _AuthenticatedWebViewer)
        self.assertIdentical(webViewer._privateApplication, self.privapp)


    def test_producePrivateRoot(self):
        """
        L{PrivateApplication.produceResource} should return a
        L{_PrivateRootPage} when asked for '/private'.
        """
        rsrc, segments = self.privapp.produceResource(FakeRequest(),
                                                     tuple(['private']), None)
        self.assertIsInstance(rsrc, _PrivateRootPage)
        self.assertEqual(segments, ())


    def test_produceRedirect(self):
        """
        L{_PrivateRootPage.produceResource} should return a redirect to
        '/private/<default-private-id>' when asked for '/'.

        This is a bad way to do it, because it isn't optional; all logged-in
        users are instantly redirected to their private page, even if the
        application has something interesting to display. See ticket #2708 for
        details.
        """
        item = FakeModelItem(store=self.userStore)
        class TabThingy(object):
            implements(INavigableElement)
            def getTabs(self):
                return [Tab("supertab", item.storeID, 1.0)]
        tt = TabThingy()
        self.userStore.inMemoryPowerUp(tt, INavigableElement)
        rsrc, segments = self.privapp.produceResource(
            FakeRequest(), tuple(['']), None)
        self.assertIsInstance(rsrc, _PrivateRootPage)
        self.assertEqual(segments, tuple(['']))
        url, newSegs = rsrc.locateChild(FakeRequest(), ('',))
        self.assertEqual(newSegs, ())
        req = FakeRequest()
        target = self.privapp.linkTo(item.storeID)
        self.assertEqual('/'+url.path, target)


    def test_produceNothing(self):
        """
        L{_PrivateRootPage.produceResource} should return None when asked for a
        resources other than '/' and '/private'.
        """
        self.assertIdentical(
            self.privapp.produceResource(FakeRequest(),
                                        tuple(['hello', 'world']),
                                        None),
            None)


    def test_privateRootHasWebViewer(self):
        """
        The L{_PrivateRootPage} returned from
        L{PrivateApplication.produceResource} should refer to an
        L{IWebViewer}.
        """
        webViewer = object()
        rsrc, segments = self.privapp.produceResource(
            FakeRequest(),
            tuple(['private']), webViewer)
        self.assertIdentical(webViewer,
                             rsrc.webViewer)
